@@ -1,5 +1,5 @@
 /* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,9 +13,9 @@
 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/backends/wince/CELauncherDialog.h,v 1.1.2.1 2004/03/03 22:27:47 arisme Exp $
+ * $Header: /cvsroot/scummvm/scummvm/backends/wince/CELauncherDialog.h,v 1.4.2.2 2006/01/18 18:07:25 sev Exp $
  *
  */
 
@@ -36,9 +36,9 @@ public:
 protected:
 	void addGame();
 	void addCandidate(String &path, DetectedGameList &candidates);
-	void automaticScanDirectory(const FilesystemNode *node);
+	void automaticScanDirectory(const FilesystemNode &node);
 };
 
 typedef GUI::LauncherDialog GUILauncherDialog;
 
-#endif
\ No newline at end of file
+#endif
