@@ -1,5 +1,5 @@
 /* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,13 +13,13 @@
 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/backends/wince/CEgui/PanelItem.cpp,v 1.2 2004/01/26 08:13:10 arisme Exp $
+ * $Header: /cvsroot/scummvm/scummvm/backends/wince/CEgui/PanelItem.cpp,v 1.6.2.2 2006/01/18 18:07:25 sev Exp $
  *
  */
 
-#include "stdafx.h"
+#include "common/stdafx.h"
 #include "PanelItem.h"
 
 namespace CEGUI {
@@ -29,7 +29,7 @@ namespace CEGUI {
 		_panel = NULL;
 	}
 
-	
+
 	PanelItem::~PanelItem() {
 	}
 
@@ -40,4 +40,5 @@ namespace CEGUI {
 	void PanelItem::setPanel(Panel *panel) {
 		_panel = panel;
 	}
-}
\ No newline at end of file
+}
+
