@@ -1,5 +1,5 @@
 /* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,16 +13,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/scumm/imuse_digi/dimuse_sndmgr.cpp,v 1.26.2.5 2004/07/18 13:49:32 kirben Exp $
+ * $Header: /cvsroot/scummvm/scummvm/scumm/imuse_digi/dimuse_sndmgr.cpp,v 1.77.2.2 2006/01/18 18:07:34 sev Exp $
  */
 
-#include "stdafx.h"
+#include "common/stdafx.h"
 #include "common/scummsys.h"
 #include "common/util.h"
+
 #include "sound/voc.h"
+#include "sound/vorbis.h"
+#include "sound/mp3.h"
+
 #include "scumm/scumm.h"
+#include "scumm/util.h"
 #include "scumm/imuse_digi/dimuse.h"
 #include "scumm/imuse_digi/dimuse_sndmgr.h"
 #include "scumm/imuse_digi/dimuse_bndmgr.h"
@@ -43,7 +48,7 @@ ImuseDigiSndMgr::~ImuseDigiSndMgr() {
 	for (int l = 0; l < MAX_IMUSE_SOUNDS; l++) {
 		closeSound(&_sounds[l]);
 	}
-#ifdef __PALM_OS__
+#ifdef PALMOS_68K
 	BundleCodecs::releaseImcTables();
 #endif
 }
@@ -79,6 +84,42 @@ void ImuseDigiSndMgr::countElements(byte *ptr, int &numRegions, int &numJumps, i
 	} while (tag != MKID_BE('DATA'));
 }
 
+void ImuseDigiSndMgr::prepareSoundFromRMAP(Common::File *file, soundStruct *sound, int32 offset, int32 size) {
+	int l;
+
+	file->seek(offset, SEEK_SET);
+	uint32 tag = file->readUint32BE();
+	assert(tag == 'RMAP');
+	int32 version = file->readUint32BE();
+	if (version != 2) {
+		error("ImuseDigiSndMgr::prepareSoundFromRMAP: Wrong version number, expected 2, but it's: %d.", version);
+	}
+	sound->bits = file->readUint32BE();
+	sound->freq = file->readUint32BE();
+	sound->channels = file->readUint32BE();
+	sound->numRegions = file->readUint32BE();
+	sound->numJumps = file->readUint32BE();
+	sound->numSyncs = file->readUint32BE();
+	sound->region = (_region *)malloc(sizeof(_region) * sound->numRegions);
+	sound->jump = (_jump *)malloc(sizeof(_jump) * sound->numJumps);
+	sound->sync = (_sync *)malloc(sizeof(_sync) * sound->numSyncs);
+	for (l = 0; l < sound->numRegions; l++) {
+		sound->region[l].offset = file->readUint32BE();
+		sound->region[l].length = file->readUint32BE();
+	}
+	for (l = 0; l < sound->numJumps; l++) {
+		sound->jump[l].offset = file->readUint32BE();
+		sound->jump[l].dest = file->readUint32BE();
+		sound->jump[l].hookId = file->readUint32BE();
+		sound->jump[l].fadeDelay = file->readUint32BE();
+	}
+	for (l = 0; l < sound->numSyncs; l++) {
+		sound->sync[l].size = file->readUint32BE();
+		sound->sync[l].ptr = (byte *)malloc(sound->sync[l].size);
+		file->read(sound->sync[l].ptr, sound->sync[l].size);
+	}
+}
+
 void ImuseDigiSndMgr::prepareSound(byte *ptr, soundStruct *sound) {
 	if (READ_UINT32(ptr) == MKID('Crea')) {
 		bool quit = false;
@@ -224,13 +265,12 @@ bool ImuseDigiSndMgr::openMusicBundle(soundStruct *sound, int disk) {
 	sound->bundle = new BundleMgr(_cacheBundleDir);
 	if (_vm->_gameId == GID_CMI) {
 		if (_vm->_features & GF_DEMO) {
-			result = sound->bundle->openFile("music.bun", _vm->getGameDataPath());
+			result = sound->bundle->open("music.bun", sound->compressed);
 		} else {
 			char musicfile[20];
 			if (disk == -1)
-				sprintf(musicfile, "musdisk%d.bun", _vm->VAR(_vm->VAR_CURRENTDISK));
-			else
-				sprintf(musicfile, "musdisk%d.bun", disk);
+				disk = _vm->VAR(_vm->VAR_CURRENTDISK);
+			sprintf(musicfile, "musdisk%d.bun", disk);
 //			if (_disk != _vm->VAR(_vm->VAR_CURRENTDISK)) {
 //				_vm->_imuseDigital->parseScriptCmds(0x1000, 0, 0, 0, 0, 0, 0, 0);
 //				_vm->_imuseDigital->parseScriptCmds(0x2000, 0, 0, 0, 0, 0, 0, 0);
@@ -238,21 +278,17 @@ bool ImuseDigiSndMgr::openMusicBundle(soundStruct *sound, int disk) {
 //				sound->bundle->closeFile();
 //			}
 
-			result = sound->bundle->openFile(musicfile, _vm->getGameDataPath());
+			result = sound->bundle->open(musicfile, sound->compressed, true);
 
-			if (result == false)
-				result = sound->bundle->openFile("music.bun", _vm->getGameDataPath());
+			// FIXME: Shouldn't we only set _disk if result == true?
 			_disk = (byte)_vm->VAR(_vm->VAR_CURRENTDISK);
 		}
 	} else if (_vm->_gameId == GID_DIG)
-		result = sound->bundle->openFile("digmusic.bun", _vm->getGameDataPath());
+		result = sound->bundle->open("digmusic.bun", sound->compressed, true);
 	else
 		error("ImuseDigiSndMgr::openMusicBundle() Don't know which bundle file to load");
 
-	if (result)
-		_vm->VAR(_vm->VAR_MUSIC_BUNDLE_LOADED) = 1;
-	else
-		_vm->VAR(_vm->VAR_MUSIC_BUNDLE_LOADED) = 0;
+	_vm->VAR(_vm->VAR_MUSIC_BUNDLE_LOADED) = result ? 1 : 0;
 
 	return result;
 }
@@ -263,13 +299,12 @@ bool ImuseDigiSndMgr::openVoiceBundle(soundStruct *sound, int disk) {
 	sound->bundle = new BundleMgr(_cacheBundleDir);
 	if (_vm->_gameId == GID_CMI) {
 		if (_vm->_features & GF_DEMO) {
-			result = sound->bundle->openFile("voice.bun", _vm->getGameDataPath());
+			result = sound->bundle->open("voice.bun", sound->compressed);
 		} else {
 			char voxfile[20];
 			if (disk == -1)
-				sprintf(voxfile, "voxdisk%d.bun", _vm->VAR(_vm->VAR_CURRENTDISK));
-			else
-				sprintf(voxfile, "voxdisk%d.bun", disk);
+				disk = _vm->VAR(_vm->VAR_CURRENTDISK);
+			sprintf(voxfile, "voxdisk%d.bun", disk);
 //			if (_disk != _vm->VAR(_vm->VAR_CURRENTDISK)) {
 //				_vm->_imuseDigital->parseScriptCmds(0x1000, 0, 0, 0, 0, 0, 0, 0);
 //				_vm->_imuseDigital->parseScriptCmds(0x2000, 0, 0, 0, 0, 0, 0, 0);
@@ -277,21 +312,17 @@ bool ImuseDigiSndMgr::openVoiceBundle(soundStruct *sound, int disk) {
 //				sound->bundle->closeFile();
 //			}
 
-			result = sound->bundle->openFile(voxfile, _vm->getGameDataPath());
+			result = sound->bundle->open(voxfile, sound->compressed);
 
-			if (result == false)
-				result = sound->bundle->openFile("voice.bun", _vm->getGameDataPath());
+			// FIXME: Shouldn't we only set _disk if result == true?
 			_disk = (byte)_vm->VAR(_vm->VAR_CURRENTDISK);
 		}
 	} else if (_vm->_gameId == GID_DIG)
-		result = sound->bundle->openFile("digvoice.bun", _vm->getGameDataPath());
+		result = sound->bundle->open("digvoice.bun", sound->compressed);
 	else
 		error("ImuseDigiSndMgr::openVoiceBundle() Don't know which bundle file to load");
 
-	if (result)
-		_vm->VAR(_vm->VAR_VOICE_BUNDLE_LOADED) = 1;
-	else
-		_vm->VAR(_vm->VAR_VOICE_BUNDLE_LOADED) = 0;
+	_vm->VAR(_vm->VAR_VOICE_BUNDLE_LOADED) = result ? 1 : 0;
 
 	return result;
 }
@@ -305,87 +336,78 @@ ImuseDigiSndMgr::soundStruct *ImuseDigiSndMgr::openSound(int32 soundId, const ch
 		error("ImuseDigiSndMgr::openSound() can't alloc free sound slot");
 	}
 
+	const bool header_outside = ((_vm->_gameId == GID_CMI) && !(_vm->_features & GF_DEMO));
 	bool result = false;
 	byte *ptr = NULL;
 
-	if (soundName[0] == 0) {
-		sound->name[0] = 0;
-		if ((soundType == IMUSE_RESOURCE)) {
-			_vm->ensureResourceLoaded(rtSound, soundId);
-			_vm->lock(rtSound, soundId);
-			ptr = _vm->getResourceAddress(rtSound, soundId);
-			if (ptr == NULL) {
-				closeSound(sound);
-				return NULL;
-			}
-			sound->resPtr = ptr;
-			sound->soundId = soundId;
-			sound->type = soundType;
-			sound->volGroupId = volGroupId;
-			result = true;
-		} else if (soundType == IMUSE_BUNDLE) {
-			bool header_outside = ((_vm->_gameId == GID_CMI) && !(_vm->_features & GF_DEMO));
-			if (volGroupId == IMUSE_VOLGRP_VOICE)
-				result = openVoiceBundle(sound, disk);
-			else if (volGroupId == IMUSE_VOLGRP_MUSIC)
-				result = openMusicBundle(sound, disk);
-			else
-				error("ImuseDigiSndMgr::openSound() Don't know how load sound: %d", soundId);
-			if (!result) {
-				closeSound(sound);
-				return NULL;
-			}
-			if (sound->bundle->decompressSampleByIndex(soundId, 0, 0x2000, &ptr, 0, header_outside) == 0) {
-				closeSound(sound);
-				return NULL;
-			}
-			sound->soundId = soundId;
-			sound->type = soundType;
-			sound->volGroupId = volGroupId;
-		} else {
-			error("ImuseDigiSndMgr::openSound() Don't know how load sound: %d", soundId);
-		}
-	} else {
-		if (soundType == IMUSE_BUNDLE) {
-			bool header_outside = ((_vm->_gameId == GID_CMI) && !(_vm->_features & GF_DEMO));
-			if (volGroupId == IMUSE_VOLGRP_VOICE)
-				result = openVoiceBundle(sound, disk);
-			else if (volGroupId == IMUSE_VOLGRP_MUSIC)
-				result = openMusicBundle(sound, disk);
-			else
-				error("ImuseDigiSndMgr::openSound() Don't know how load sound: %d", soundId);
-			if (!result) {
-				closeSound(sound);
-				return NULL;
-			}
-			if (sound->bundle->decompressSampleByName(soundName, 0, 0x2000, &ptr, header_outside) == 0) {
-				closeSound(sound);
-				return NULL;
-			}
-			strcpy(sound->name, soundName);
-			sound->soundId = soundId;
-			sound->type = soundType;
-			sound->volGroupId = volGroupId;
-		} else {
-			error("ImuseDigiSndMgr::openSound() Don't know how load sound: %s", soundName);
-		}
-	}
+	switch (soundType) {
+	case IMUSE_RESOURCE:
+		assert(soundName[0] == 0);	// Paranoia check
 
-	if (result) {
+		_vm->ensureResourceLoaded(rtSound, soundId);
+		_vm->res.lock(rtSound, soundId);
+		ptr = _vm->getResourceAddress(rtSound, soundId);
 		if (ptr == NULL) {
 			closeSound(sound);
 			return NULL;
 		}
-		sound->disk = _disk;
-		prepareSound(ptr, sound);
-		return sound;
+		sound->resPtr = ptr;
+		break;
+	case IMUSE_BUNDLE:
+		if (volGroupId == IMUSE_VOLGRP_VOICE)
+			result = openVoiceBundle(sound, disk);
+		else if (volGroupId == IMUSE_VOLGRP_MUSIC)
+			result = openMusicBundle(sound, disk);
+		else
+			error("ImuseDigiSndMgr::openSound() Don't know how load sound: %d", soundId);
+		if (!result) {
+			closeSound(sound);
+			return NULL;
+		}
+		if (sound->compressed) {
+			char fileName[24];
+			int32 offset = 0, size = 0;
+			sprintf(fileName, "%s.map", soundName);
+			Common::File *rmapFile = sound->bundle->getFile(fileName, offset, size);
+			if (!rmapFile) {
+				closeSound(sound);
+				return NULL;
+			}
+			prepareSoundFromRMAP(rmapFile, sound, offset, size);
+			strcpy(sound->name, soundName);
+			sound->soundId = soundId;
+			sound->type = soundType;
+			sound->volGroupId = volGroupId;
+			sound->disk = _disk;
+			return sound;
+		} else if (soundName[0] == 0) {
+			if (sound->bundle->decompressSampleByIndex(soundId, 0, 0x2000, &ptr, 0, header_outside) == 0 || ptr == NULL) {
+				closeSound(sound);
+				return NULL;
+			}
+		} else {
+			if (sound->bundle->decompressSampleByName(soundName, 0, 0x2000, &ptr, header_outside) == 0 || ptr == NULL) {
+				closeSound(sound);
+				return NULL;
+			}
+		}
+		sound->resPtr = 0;
+		break;
+	default:
+		error("ImuseDigiSndMgr::openSound() Unknown soundType %d (trying to load sound %d)", soundType, soundId);
 	}
 
-	return NULL;
+	strcpy(sound->name, soundName);
+	sound->soundId = soundId;
+	sound->type = soundType;
+	sound->volGroupId = volGroupId;
+	sound->disk = _disk;
+	prepareSound(ptr, sound);
+	return sound;
 }
 
 void ImuseDigiSndMgr::closeSound(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 
 	if (soundHandle->resPtr) {
 		bool found = false;
@@ -394,10 +416,14 @@ void ImuseDigiSndMgr::closeSound(soundStruct *soundHandle) {
 				found = true;
 		}
 		if (!found)
-			_vm->unlock(rtSound, soundHandle->soundId);
+			_vm->res.unlock(rtSound, soundHandle->soundId);
 	}
 
+	if (soundHandle->compressedStream)
+		delete soundHandle->compressedStream;
+
 	delete soundHandle->bundle;
+
 	for (int r = 0; r < soundHandle->numSyncs; r++)
 		free(soundHandle->sync[r].ptr);
 	free(soundHandle->region);
@@ -407,12 +433,14 @@ void ImuseDigiSndMgr::closeSound(soundStruct *soundHandle) {
 }
 
 ImuseDigiSndMgr::soundStruct *ImuseDigiSndMgr::cloneSound(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 
 	return openSound(soundHandle->soundId, soundHandle->name, soundHandle->type, soundHandle->volGroupId, soundHandle->disk);
 }
 
 bool ImuseDigiSndMgr::checkForProperHandle(soundStruct *soundHandle) {
+	if (!soundHandle)
+		return false;
 	for (int l = 0; l < MAX_IMUSE_SOUNDS; l++) {
 		if (soundHandle == &_sounds[l])
 			return true;
@@ -420,47 +448,52 @@ bool ImuseDigiSndMgr::checkForProperHandle(soundStruct *soundHandle) {
 	return false;
 }
 
+bool ImuseDigiSndMgr::isCompressed(soundStruct *soundHandle) {
+	assert(checkForProperHandle(soundHandle));
+	return soundHandle->compressed;
+}
+
 int ImuseDigiSndMgr::getFreq(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	return soundHandle->freq;
 }
 
 int ImuseDigiSndMgr::getBits(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	return soundHandle->bits;
 }
 
 int ImuseDigiSndMgr::getChannels(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	return soundHandle->channels;
 }
 
 bool ImuseDigiSndMgr::isEndOfRegion(soundStruct *soundHandle, int region) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(region >= 0 && region < soundHandle->numRegions);
 	return soundHandle->endFlag;
 }
 
 int ImuseDigiSndMgr::getNumRegions(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	return soundHandle->numRegions;
 }
 
 int ImuseDigiSndMgr::getNumJumps(soundStruct *soundHandle) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	return soundHandle->numJumps;
 }
 
 int ImuseDigiSndMgr::getRegionOffset(soundStruct *soundHandle, int region) {
-	debug(5, "getRegionOffset() region:%d");
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	debug(5, "getRegionOffset() region:%d", region);
+	assert(checkForProperHandle(soundHandle));
 	assert(region >= 0 && region < soundHandle->numRegions);
 	return soundHandle->region[region].offset;
 }
 
 int ImuseDigiSndMgr::getJumpIdByRegionAndHookId(soundStruct *soundHandle, int region, int hookId) {
 	debug(5, "getJumpIdByRegionAndHookId() region:%d, hookId:%d", region, hookId);
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(region >= 0 && region < soundHandle->numRegions);
 	int32 offset = soundHandle->region[region].offset;
 	for (int l = 0; l < soundHandle->numJumps; l++) {
@@ -469,12 +502,12 @@ int ImuseDigiSndMgr::getJumpIdByRegionAndHookId(soundStruct *soundHandle, int re
 				return l;
 		}
 	}
-	
+
 	return -1;
 }
 
 void ImuseDigiSndMgr::getSyncSizeAndPtrById(soundStruct *soundHandle, int number, int32 &sync_size, byte **sync_ptr) {
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(number >= 0);
 	if (number < soundHandle->numSyncs) {
 		sync_size = soundHandle->sync[number].size;
@@ -487,7 +520,7 @@ void ImuseDigiSndMgr::getSyncSizeAndPtrById(soundStruct *soundHandle, int number
 
 int ImuseDigiSndMgr::getRegionIdByJumpId(soundStruct *soundHandle, int jumpId) {
 	debug(5, "getRegionIdByJumpId() jumpId:%d", jumpId);
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(jumpId >= 0 && jumpId < soundHandle->numJumps);
 	int32 dest = soundHandle->jump[jumpId].dest;
 	for (int l = 0; l < soundHandle->numRegions; l++) {
@@ -501,21 +534,21 @@ int ImuseDigiSndMgr::getRegionIdByJumpId(soundStruct *soundHandle, int jumpId) {
 
 int ImuseDigiSndMgr::getJumpHookId(soundStruct *soundHandle, int number) {
 	debug(5, "getJumpHookId() number:%d", number);
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(number >= 0 && number < soundHandle->numJumps);
 	return soundHandle->jump[number].hookId;
 }
 
 int ImuseDigiSndMgr::getJumpFade(soundStruct *soundHandle, int number) {
 	debug(5, "getJumpFade() number:%d", number);
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(number >= 0 && number < soundHandle->numJumps);
 	return soundHandle->jump[number].fadeDelay;
 }
 
 int32 ImuseDigiSndMgr::getDataFromRegion(soundStruct *soundHandle, int region, byte **buf, int32 offset, int32 size) {
 	debug(5, "getDataFromRegion() region:%d, offset:%d, size:%d, numRegions:%d", region, offset, size, soundHandle->numRegions);
-	assert(soundHandle && checkForProperHandle(soundHandle));
+	assert(checkForProperHandle(soundHandle));
 	assert(buf && offset >= 0 && size >= 0);
 	assert(region >= 0 && region < soundHandle->numRegions);
 
@@ -533,13 +566,56 @@ int32 ImuseDigiSndMgr::getDataFromRegion(soundStruct *soundHandle, int region, b
 
 	int header_size = soundHandle->offsetData;
 	bool header_outside = ((_vm->_gameId == GID_CMI) && !(_vm->_features & GF_DEMO));
-	if (soundHandle->bundle) {
+	if ((soundHandle->bundle) && (!soundHandle->compressed)) {
 		size = soundHandle->bundle->decompressSampleByCurIndex(start + offset, size, buf, header_size, header_outside);
 	} else if (soundHandle->resPtr) {
 		*buf = (byte *)malloc(size);
 		memcpy(*buf, soundHandle->resPtr + start + offset + header_size, size);
+	} else if ((soundHandle->bundle) && (soundHandle->compressed)) {
+		*buf = (byte *)malloc(size);
+		char fileName[24];
+		sprintf(fileName, "%s_reg%03d", soundHandle->name, region);
+		if (scumm_stricmp(fileName, soundHandle->lastFileName) != 0) {
+			int32 offs = 0, len = 0;
+			Common::File *cmpFile;
+			bool oggMode = false;
+			sprintf(fileName, "%s_reg%03d.mp3", soundHandle->name, region);
+			cmpFile = soundHandle->bundle->getFile(fileName, offs, len);
+#ifndef USE_MAD
+			if (len)
+				error("Mad library compiled support needed!");
+#endif
+			if (!len) {
+				sprintf(fileName, "%s_reg%03d.ogg", soundHandle->name, region);
+				cmpFile = soundHandle->bundle->getFile(fileName, offs, len);
+#ifndef USE_VORBIS
+				if (len)
+					error("Vorbis library compiled support needed!");
+#endif
+				assert(len);
+				oggMode = true;
+			}
+			if (!soundHandle->compressedStream) {
+#ifdef USE_VORBIS
+				if (oggMode)
+					soundHandle->compressedStream = makeVorbisStream(cmpFile, len);
+#endif
+#ifdef USE_MAD
+				if (!oggMode)
+					soundHandle->compressedStream = makeMP3Stream(cmpFile, len);
+#endif
+				assert(soundHandle->compressedStream);
+			}
+			strcpy(soundHandle->lastFileName, fileName);
+		}
+		size = soundHandle->compressedStream->readBuffer((int16 *)*buf, size / 2) * 2;
+		if (soundHandle->compressedStream->endOfData()) {
+			delete soundHandle->compressedStream;
+			soundHandle->compressedStream = NULL;
+			soundHandle->lastFileName[0] = 0;
+		}
 	}
-	
+
 	return size;
 }
 
