@@ -1,5 +1,5 @@
 /* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,17 +13,17 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/scumm/imuse_digi/dimuse_tables.cpp,v 1.14.2.1 2004/05/02 16:36:04 aquadran Exp $
+ * $Header: /cvsroot/scummvm/scummvm/scumm/imuse_digi/dimuse_tables.cpp,v 1.22.2.2 2006/01/18 18:07:34 sev Exp $
  */
 
-#include "stdafx.h"
+#include "common/stdafx.h"
 #include "scumm/imuse_digi/dimuse.h"
 
 namespace Scumm {
 
-#ifdef __PALM_OS__
+#ifdef PALMOS_68K
 const imuseRoomMap *_digStateMusicMap;
 const imuseDigTable *_digStateMusicTable;
 const imuseDigTable *_digSeqMusicTable;
@@ -852,7 +852,7 @@ const imuseFtSeqTable _ftSeqMusicTable[] = {
 
 } // End of namespace Scumm
 
-#ifdef __PALM_OS__
+#ifdef PALMOS_68K
 #include "scumm_globals.h"
 
 _GINIT(DimuseTables)
