@@ -1,6 +1,6 @@
 /* ScummVM - Scumm Interpreter
  * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -14,16 +14,18 @@
 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/scumm/insane/insane.cpp,v 1.19.2.5 2004/06/26 09:37:27 aquadran Exp $
+ * $Header: /cvsroot/scummvm/scummvm/scumm/insane/insane.cpp,v 1.66.2.2 2006/01/18 18:07:34 sev Exp $
  *
  */
 
-#include "stdafx.h"
+#include "common/stdafx.h"
 
 #include "base/engine.h"
 
+#include "common/system.h"
+
 #include "scumm/scumm.h"
 #include "scumm/actor.h"
 #include "scumm/sound.h"
@@ -40,27 +42,19 @@
 
 // TODO (in no particular order):
 // o Code review/cleanup
-// o DOS demo INSANE
 
 namespace Scumm {
 
-static const int actorAnimationData[21] = {20, 21, 22, 23, 24, 25, 26, 13, 14, 15, 16, 17, 
+static const int actorAnimationData[21] = {20, 21, 22, 23, 24, 25, 26, 13, 14, 15, 16, 17,
 	18, 19, 6, 7, 8, 9, 10, 11, 12};
 
 
 Insane::Insane(ScummEngine_v6 *scumm) {
 	_vm = scumm;
-	
-#ifndef FTDOSDEMO
-	if ((_vm->_features & GF_DEMO) && (_vm->_features & GF_PC)) {
-		_insaneIsRunning = false;
-		return;
-	}
-#endif
 
 	initvars();
 
-	if (!((_vm->_features & GF_DEMO) && (_vm->_features & GF_PC))) {
+	if (!((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))) {
 		readFileToMem("roadrash.rip", &_smush_roadrashRip);
 		readFileToMem("roadrsh2.rip", &_smush_roadrsh2Rip);
 		readFileToMem("roadrsh3.rip", &_smush_roadrsh3Rip);
@@ -71,15 +65,15 @@ Insane::Insane(ScummEngine_v6 *scumm) {
 		readFileToMem("minedriv.flu", &_smush_minedrivFlu);
 		readFileToMem("minefite.flu", &_smush_minefiteFlu);
 		_smush_bensgoggNut = new NutRenderer(_vm);
-		_smush_bensgoggNut->loadFont("bensgogg.nut", _vm->getGameDataPath());
+		_smush_bensgoggNut->loadFont("bensgogg.nut");
 		_smush_bencutNut = new NutRenderer(_vm);
-		_smush_bencutNut->loadFont("bencut.nut", _vm->getGameDataPath());
+		_smush_bencutNut->loadFont("bencut.nut");
 	}
 
 	_smush_iconsNut = new NutRenderer(_vm);
-	_smush_iconsNut->loadFont("icons.nut", _vm->getGameDataPath());
+	_smush_iconsNut->loadFont("icons.nut");
 	_smush_icons2Nut = new NutRenderer(_vm);
-	_smush_icons2Nut->loadFont("icons2.nut", _vm->getGameDataPath());
+	_smush_icons2Nut->loadFont("icons2.nut");
 }
 
 Insane::~Insane(void) {
@@ -130,7 +124,6 @@ void Insane::initvars(void) {
 	_keyboardDisable = 0;
 	_needSceneSwitch = false;
 	_idx2Exceeded = 0;
-	_memoryAllocatedNotOK = 0;
 	_lastKey = 0;
 	_tiresRustle = false;
 	_keybOldDx = 0;
@@ -164,17 +157,19 @@ void Insane::initvars(void) {
 	_roadBumps = false;
 	_val211d = 0;
 	_val213d = 0;
-	_val215d = 0;
+	_metEnemiesListTail = 0;
 	_smlayer_room = 0;
 	_smlayer_room2 = 0;
 	_isBenCut = 0;
-	_smush_isPauseImuse = false;
 	_continueFrame = 0;
 	_continueFrame1 = 0;
 	_counter1 = 0;
 	_iactSceneId = 0;
 	_iactSceneId2 = 0;
 
+	for (i = 0; i < 12; i++)
+		_metEnemiesList[i] = 0;
+
 	for (i = 0; i < 9; i++)
 		for (j = 0; j < 9; j++)
 			_enHdlVar[i][j] = 0;
@@ -182,23 +177,30 @@ void Insane::initvars(void) {
 	for (i = 0; i < 0x80; i++)
 		_iactBits[i] = 0;
 
-	init_enemyStruct(EN_ROTT1, EN_ROTT1, 0, 0, 160, 0, INV_MACE, 90, "wr2_rott.san", 
-					 26, 16, 17, 27, 11, 3);
-	init_enemyStruct(EN_ROTT2, EN_ROTT2, 1, 0, 250, 0, INV_2X4, 90, "wr2_rott.san", 
+
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+		init_enemyStruct(EN_ROTT1, EN_ROTT1, 0, 0, 160, 0, INV_MACE, 63, "endcrshr.san",
+						 25, 15, 16, 26, 11, 3);
+	} else {
+		init_enemyStruct(EN_ROTT1, EN_ROTT1, 0, 0, 160, 0, INV_MACE, 90, "wr2_rott.san",
+						 26, 16, 17, 27, 11, 3);
+	}
+
+	init_enemyStruct(EN_ROTT2, EN_ROTT2, 1, 0, 250, 0, INV_2X4, 90, "wr2_rott.san",
 					 28, 16, 17, 42, 11, 3);
-	init_enemyStruct(EN_ROTT3, EN_ROTT3, 2, 0, 120, 0, INV_HAND, 90, "wr2_rott.san", 
+	init_enemyStruct(EN_ROTT3, EN_ROTT3, 2, 0, 120, 0, INV_HAND, 90, "wr2_rott.san",
 					 15, 16, 17, 43, 11, 3);
-	init_enemyStruct(EN_VULTF1, EN_VULTF1, 3, 0, 60, 0, INV_HAND, 91, "wr2_vltp.san", 
+	init_enemyStruct(EN_VULTF1, EN_VULTF1, 3, 0, 60, 0, INV_HAND, 91, "wr2_vltp.san",
 					 29, 33, 32, 37, 12, 4);
-	init_enemyStruct(EN_VULTM1, EN_VULTM1, 4, 0, 100, 0, INV_CHAIN, 91, "wr2_vltc.san", 
+	init_enemyStruct(EN_VULTM1, EN_VULTM1, 4, 0, 100, 0, INV_CHAIN, 91, "wr2_vltc.san",
 					 30, 33, 32, 36, 12, 4);
-	init_enemyStruct(EN_VULTF2, EN_VULTF2, 5, 0, 250, 0, INV_CHAINSAW, 91, "wr2_vlts.san", 
+	init_enemyStruct(EN_VULTF2, EN_VULTF2, 5, 0, 250, 0, INV_CHAINSAW, 91, "wr2_vlts.san",
 					 31, 33, 32, 35, 12, 4);
-	init_enemyStruct(EN_VULTM2, EN_VULTM2, 6, 0, 900, 0, INV_BOOT, 91, "wr2_rott.san", 
+	init_enemyStruct(EN_VULTM2, EN_VULTM2, 6, 0, 900, 0, INV_BOOT, 91, "wr2_rott.san",
 					 34, 33, 32, 45, 16, 4);
-	init_enemyStruct(EN_CAVEFISH, EN_CAVEFISH, 7, 0, 60, 0, INV_DUST, 92, "wr2_cave.san", 
+	init_enemyStruct(EN_CAVEFISH, EN_CAVEFISH, 7, 0, 60, 0, INV_DUST, 92, "wr2_cave.san",
 					 39, 0, 0, 41, 13, 2);
-	init_enemyStruct(EN_TORQUE, EN_TORQUE, 8, 0, 900, 0, INV_HAND, 93, "wr2_vltp.san", 
+	init_enemyStruct(EN_TORQUE, EN_TORQUE, 8, 0, 900, 0, INV_HAND, 93, "wr2_vltp.san",
 					 57, 0, 0, 37, 12, 1);
 
 	init_fluConfStruct(1, 1, &_smush_minedrivFlu, "minedriv.san", 235, 1300);
@@ -217,7 +219,7 @@ void Insane::initvars(void) {
 	init_fluConfStruct(18, 4, &_smush_tovista1Flu, "tovista1.san", 175, 230);
 	init_fluConfStruct(19, 6, &_smush_toranchFlu, "toranch.san", 115, 530);
 	init_fluConfStruct(20, 6, &_smush_toranchFlu, "toranch.san", 115, 530);
-  
+
 	init_scenePropStruct(  0,  0, 0, 0, 0, 0, 0, 0, 0, 0, 1);
 	init_scenePropStruct(  1,  0, 1, 128, 2001, 0, 0, 0, 0, 56, 2);
 	init_scenePropStruct(  2,  0, 0, 125, 1002, 0, 0, 0, 0, 35, 3);
@@ -371,8 +373,8 @@ void Insane::initvars(void) {
 	_actor[0].x1 = -1;
 	_actor[0].weaponClass = 2;
 	_actor[0].animWeaponClass = 0;
-	_actor[0].field_34 = 2;
-	_actor[0].field_38 = 0;
+	_actor[0].newFacingFlag = 2;
+	_actor[0].curFacingFlag = 0;
 	_actor[0].lost = false;
 	_actor[0].kicking = false;
 	_actor[0].field_44 = false;
@@ -410,8 +412,8 @@ void Insane::initvars(void) {
 	_actor[1].x1 = -1;
 	_actor[1].weaponClass = 2;
 	_actor[1].animWeaponClass = 0;
-	_actor[1].field_34 = 0;
-	_actor[1].field_38 = 0;
+	_actor[1].newFacingFlag = 0;
+	_actor[1].curFacingFlag = 0;
 	_actor[1].lost = false;
 	_actor[1].kicking = false;
 	_actor[1].field_44 = false;
@@ -442,7 +444,7 @@ void Insane::initvars(void) {
 			_enemyState[i][j] = 0;
 }
 
-void Insane::init_actStruct(int actornum, int actnum, int32 actorval, byte state, 
+void Insane::init_actStruct(int actornum, int actnum, int32 actorval, byte state,
 								  int32 room, int32 animTilt, int32 tilt, int32 frame) {
 	_actor[actornum].act[actnum].actor = actorval;
 	_actor[actornum].act[actnum].state = state;
@@ -453,7 +455,7 @@ void Insane::init_actStruct(int actornum, int actnum, int32 actorval, byte state
 }
 
 void Insane::init_enemyStruct(int n, int32 handler, int32 initializer,
-								   int16 occurences, int32 maxdamage, int32 field_10,
+								   int16 occurences, int32 maxdamage, int32 isEmpty,
 								   int32 weapon, int32 sound, const char *filename,
 								   int32 costume4, int32 costume6, int32 costume5,
 								   int16 costumevar, int32 maxframe, int32 apprAnim) {
@@ -463,7 +465,7 @@ void Insane::init_enemyStruct(int n, int32 handler, int32 initializer,
 	_enemy[n].initializer = initializer;
 	_enemy[n].occurences = occurences;
 	_enemy[n].maxdamage = maxdamage;
-	_enemy[n].field_10 = field_10;
+	_enemy[n].isEmpty = isEmpty;
 	_enemy[n].weapon = weapon;
 	_enemy[n].sound = sound;
 	strncpy(_enemy[n].filename, filename, 20);
@@ -475,7 +477,7 @@ void Insane::init_enemyStruct(int n, int32 handler, int32 initializer,
 	_enemy[n].apprAnim = apprAnim;
 }
 
-void Insane::init_fluConfStruct(int n, int sceneId, byte **fluPtr, 
+void Insane::init_fluConfStruct(int n, int sceneId, byte **fluPtr,
 					const char *filenamePtr, int startFrame, int numFrames) {
 	_fluConf[n].sceneId = sceneId;
 	_fluConf[n].fluPtr = fluPtr;
@@ -500,7 +502,7 @@ void Insane::init_scenePropStruct(int32 n, int32 n1, int32 actornum, int32 sound
 
 void Insane::setBenAnimation(int32 actornum, int anim) {
 	if (anim <= 12)
-		smlayer_setActorFacing(actornum, 1, 
+		smlayer_setActorFacing(actornum, 1,
 			  actorAnimationData[_actor[actornum].weaponClass * 7 + anim - 6], 180);
 }
 
@@ -511,8 +513,8 @@ void Insane::setEnemyAnimation(int32 actornum, int anim) {
 		d = 14;
 
 	if (anim <= 12)
-		smlayer_setActorFacing(actornum, 1, 
-			  actorAnimationData[_actor[actornum].weaponClass * 7 + anim - 6 + d], 180);
+		smlayer_setActorFacing(actornum, 1,
+			  actorAnimationData[_actor[actornum].weaponClass * 7 + anim - 6] + d, 180);
 }
 
 int32 Insane::processMouse(void) {
@@ -587,18 +589,16 @@ int32 Insane::processKeyboard(void) {
 }
 
 void Insane::readFileToMem(const char *name, byte **buf) {
-	File *in;
+	ScummFile in;
 	uint32 len;
 
-	in = new File();
-	in->open(name, _vm->getGameDataPath());
-	len = in->size();
+	_vm->openFile(in, name);
+	len = in.size();
 	*buf = (byte *)malloc(len);
-	in->read(*buf, len);
-	in->close();
+	in.read(*buf, len);
 }
 
-void Insane::startVideo(const char *filename, int num, int argC, int frameRate, 
+void Insane::startVideo(const char *filename, int num, int argC, int frameRate,
 						 int doMainLoop, byte *fluPtr, int32 startFrame) {
 	int32 offset = 0;
 
@@ -615,11 +615,11 @@ void Insane::startVideo(const char *filename, int num, int argC, int frameRate,
 		smush_setupSanFromStart(filename, 0, -1, -1, 0);
 	}
 
-	_player->play(filename, _vm->getGameDataPath(), offset, startFrame);
+	_player->play(filename, offset, startFrame);
 }
 
 void Insane::smush_warpMouse(int x, int y, int buttons) {
-	_vm->_system->warp_mouse(x, y);
+	_player->warpMouse(x, y, buttons);
 }
 
 void Insane::putActors(void) {
@@ -632,36 +632,79 @@ void Insane::putActors(void) {
 }
 
 void Insane::readState(void) { // PATCH
-	_actor[0].inventory[INV_CHAIN] = readArray(50) != 0;
-	_actor[0].inventory[INV_CHAINSAW] = readArray(51) != 0;
-	_actor[0].inventory[INV_MACE] = readArray(52) != 0;
-	_actor[0].inventory[INV_2X4] = readArray(53) != 0;
-	_actor[0].inventory[INV_WRENCH] = readArray(54) != 0;
-	_actor[0].inventory[INV_DUST] = readArray(55) != 0;
-	_actor[0].inventory[INV_HAND] = 1;
-	_actor[0].inventory[INV_BOOT] = 1;
 
-	_smlayer_room = readArray(320);
-	_smlayer_room2 = readArray(321);
-	_posBrokenTruck = readArray(322);
-	_posVista = readArray(323);
-	_val57d = readArray(324);
-	_posCave = readArray(325);
-	_posBrokenCar = readArray(326);
-	_val54d = readArray(327);
-	_posFatherTorque = readArray(328);
-	_enemy[EN_TORQUE].occurences = readArray(337);
-	_enemy[EN_ROTT1].occurences = readArray(329);
-	_enemy[EN_ROTT2].occurences = readArray(330);
-	_enemy[EN_ROTT3].occurences = readArray(331);
-	_enemy[EN_VULTF1].occurences = readArray(332);
-	_enemy[EN_VULTM1].occurences = readArray(333);
-	_enemy[EN_VULTF2].occurences = readArray(334);
-	_enemy[EN_VULTM2].occurences = readArray(335);
-	_enemy[EN_CAVEFISH].occurences = readArray(336);
-	_enemy[EN_VULTM2].field_10 = readArray(340);
-	_enemy[EN_CAVEFISH].field_10 = readArray(56);
-	_enemy[EN_VULTF2].field_10 = readArray(339);
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+		_actor[0].inventory[INV_CHAIN] = 0;
+		_actor[0].inventory[INV_CHAINSAW] = 0;
+		_actor[0].inventory[INV_MACE] = 0;
+		_actor[0].inventory[INV_2X4] = 0;
+		_actor[0].inventory[INV_WRENCH] = 1;
+		_actor[0].inventory[INV_DUST] = 0;
+		_actor[0].inventory[INV_HAND] = 1;
+		_actor[0].inventory[INV_BOOT] = 0;
+		_smlayer_room2 = 13;
+	} else {
+		_actor[0].inventory[INV_CHAIN] = readArray(50) != 0;
+		_actor[0].inventory[INV_CHAINSAW] = readArray(51) != 0;
+		_actor[0].inventory[INV_MACE] = readArray(52) != 0;
+		_actor[0].inventory[INV_2X4] = readArray(53) != 0;
+		_actor[0].inventory[INV_WRENCH] = readArray(54) != 0;
+		_actor[0].inventory[INV_DUST] = readArray(55) != 0;
+		_actor[0].inventory[INV_HAND] = 1;
+		_actor[0].inventory[INV_BOOT] = 1;
+		_smlayer_room = readArray(320);
+		_smlayer_room2 = readArray(321);
+		_posBrokenTruck = readArray(322);
+		_posVista = readArray(323);
+		_val57d = readArray(324);
+		_posCave = readArray(325);
+		_posBrokenCar = readArray(326);
+		_val54d = readArray(327);
+		_posFatherTorque = readArray(328);
+		_enemy[EN_TORQUE].occurences = readArray(337);
+		_enemy[EN_ROTT1].occurences = readArray(329);
+		_enemy[EN_ROTT2].occurences = readArray(330);
+		_enemy[EN_ROTT3].occurences = readArray(331);
+		_enemy[EN_VULTF1].occurences = readArray(332);
+		_enemy[EN_VULTM1].occurences = readArray(333);
+		_enemy[EN_VULTF2].occurences = readArray(334);
+		_enemy[EN_VULTM2].occurences = readArray(335);
+		_enemy[EN_CAVEFISH].occurences = readArray(336);
+		_enemy[EN_VULTM2].isEmpty = readArray(340);
+		_enemy[EN_VULTF2].isEmpty = readArray(339);
+		_enemy[EN_CAVEFISH].isEmpty = readArray(56);
+
+		// FIXME
+		// Some sanity checks. There were submitted savefiles where these values were wrong
+		// Still it is unknown what leads to this state. Most probably it is memory
+		// overwrite
+		if (_enemy[EN_VULTM2].isEmpty != readArray(7)) {
+			warning("Wrong INSANE parameters for EN_VULTM2 (%d %d)",
+				  _enemy[EN_VULTM2].isEmpty, readArray(7));
+			_enemy[EN_VULTM2].isEmpty = readArray(7);
+		}
+
+		if (_enemy[EN_VULTF2].isEmpty != (_actor[0].inventory[INV_CHAINSAW] != 0)) {
+			warning("Wrong INSANE parameters for EN_VULTF2 (%d %d)",
+				  _enemy[EN_VULTF2].isEmpty, _actor[0].inventory[INV_CHAINSAW]);
+			_enemy[EN_VULTF2].isEmpty = (_actor[0].inventory[INV_CHAINSAW] != 0);
+		}
+
+		// FIXME
+		// This used to be here but.
+		//  - bootparam 551 gives googles without cavefish met
+		//  - when you get the ramp, googles disappear, but you already won the cavefish
+		// Incorrect situation would be
+		//  you won cavefish, don't have googles, don't have ramp
+		//
+		// So if you find out what how to check ramp presense, feel free to add check here
+		// (beware of FT ver a and ver b. In version b var311 is inserted and all vars >311
+		// are shifted),
+		//
+		//if (_enemy[EN_CAVEFISH].isEmpty != readArray(8))
+		//	error("Wrong INSANE parameters for EN_CAVEFISH (%d %d). Please, report this",
+		//		  _enemy[EN_CAVEFISH].isEmpty, readArray(8));
+	}
 }
 
 void Insane::setupValues(void) {
@@ -683,47 +726,6 @@ void Insane::setupValues(void) {
 	smush_warpMouse(160, 100, -1);
 }
 
-// FIXME: it seems that in ScummVM it may be unused
-void Insane::mainLoop(void) {
-	int32 resid;
-
-	while (!idx2Compare()) {
-		if(!(resid = idx2Tweak()))
-			continue;
-	  
-		_vm->ensureResourceLoaded(rtSound, resid);
-		_vm->setResourceCounter(rtSound, resid, 1);
-	}
-	_vm->increaseResourceCounter();
-	
-	while (!idx1Compare()) {
-		resid = idx1Tweak();
-		_vm->ensureResourceLoaded(rtCostume, resid);
-		_vm->setResourceCounter(rtCostume, resid, 1);
-		// smlayer_lock (rtCostume, resid);
-	}
-	
-	if (loadSceneData(_currSceneId, 0, 2)) {
-		_sceneData1Loaded = 0;
-		_objArray2[0] = 0;
-		return;
-	}
-	_sceneData1Loaded = 1;
-	if (idx1Compare() || idx2Compare()) {
-		_objArray2[0] = 0;
-		return;
-	}
-	_objArray2[0]++;
-	if (_objArray2[0] <= 5) {
-		_objArray2[0] = 0;
-		return;
-	}
-	
-	smush_setToFinish();
-	loadSceneData(_currSceneId, 1, 2);
-	_memoryAllocatedNotOK = 1;
-}
-
 bool Insane::idx1Compare(void) {
 	return _objArray1Idx == _objArray1Idx2;
 }
@@ -736,7 +738,7 @@ int32 Insane::idx1Tweak(void) {
 	_objArray1Idx++;
 	if (_objArray1Idx >= 100)
 		_objArray1Idx = 0;
-	
+
 	return _objArray1[_objArray1Idx];
 }
 
@@ -744,7 +746,7 @@ int32 Insane::idx2Tweak(void) {
 	if (!_idx2Exceeded)
 		if (_objArray2Idx >= _objArray2Idx2)
 			return false;
-	
+
 	_objArray2Idx++;
 	if (_objArray2Idx >= 100) {
 		_idx2Exceeded = 0;
@@ -754,8 +756,8 @@ int32 Insane::idx2Tweak(void) {
 }
 
 void Insane::smush_setToFinish(void) {
-	debug(INSANE_DBG, "Video is set to finish");
-	_vm->_videoFinished = 1;
+	debugC(DEBUG_INSANE, "Video is set to finish");
+	_vm->_smushVideoShouldFinish = true;
 }
 
 // smlayer_stopSound
@@ -772,7 +774,7 @@ void Insane::switchSceneIfNeeded(void) {
 		_needSceneSwitch = false;
 		loadSceneData(_temp2SceneId, 0, 1);
 
-		if(loadSceneData(_temp2SceneId, 0, 2)) {
+		if (loadSceneData(_temp2SceneId, 0, 2)) {
 			setSceneCostumes(_temp2SceneId);
 			_sceneData2Loaded = 0;
 			_sceneData1Loaded = 0;
@@ -798,10 +800,10 @@ void Insane::prepareScenePropScene(int32 scenePropNum, bool arg_4, bool arg_8) {
 
 	int tmp, idx = scenePropIdx[scenePropNum];
 
-	debug(INSANE_DBG, "Insane::prepareScenePropScene(%d, %d, %d)", scenePropNum, arg_4, arg_8);
+	debugC(DEBUG_INSANE, "Insane::prepareScenePropScene(%d, %d, %d)", scenePropNum, arg_4, arg_8);
 
-	if (!loadScenePropSounds(idx))
-		return;
+	if (((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) || !loadScenePropSounds(idx))
+			return;
 
 	_actor[0].defunct = arg_4;
 	_actor[1].defunct = arg_8;
@@ -836,20 +838,20 @@ int Insane::smush_changeState(int state) {
 	return _smush_smushState;
 }
 
-void Insane::queueSceneSwitch(int32 sceneId, byte *fluPtr, const char *filename, 
+void Insane::queueSceneSwitch(int32 sceneId, byte *fluPtr, const char *filename,
 							  int32 arg_C, int32 arg_10, int32 startFrame, int32 numFrames) {
 	int32 tmp;
-	
-	debug(INSANE_DBG, "queueSceneSwitch(%d, *, %s, %d, %d, %d, %d)", sceneId, filename, arg_C, arg_10,
+
+	debugC(DEBUG_INSANE, "queueSceneSwitch(%d, *, %s, %d, %d, %d, %d)", sceneId, filename, arg_C, arg_10,
 		  startFrame, numFrames);
 	if (_needSceneSwitch || _sceneData1Loaded)
 		return;
-	
+
 	if (fluPtr) {
 		tmp = ((int)startFrame/30+1)*30;
 		if (tmp >= numFrames)
 			tmp = 0;
-	
+
 		smush_setupSanWithFlu(filename, arg_C|32, -1, -1, 0, fluPtr, tmp);
 	} else {
 		smush_setupSanFromStart(filename, arg_C|32, -1, -1, 0);
@@ -859,10 +861,10 @@ void Insane::queueSceneSwitch(int32 sceneId, byte *fluPtr, const char *filename,
 }
 
 void Insane::smush_rewindCurrentSan(int arg_0, int arg_4, int arg_8) {
-	debug(INSANE_DBG, "smush_rewindCurrentSan(%d, %d, %d)", arg_0, arg_4, arg_8);
+	debugC(DEBUG_INSANE, "smush_rewindCurrentSan(%d, %d, %d)", arg_0, arg_4, arg_8);
 	_smush_setupsan2 = arg_0;
-	
-	smush_setupSanFile(0, 8, 0);
+
+	smush_setupSanFile(0, 0, 0);
 	_smush_isSanFileSetup = 1;
 	smush_setFrameSteps(arg_4, arg_8);
 
@@ -874,7 +876,7 @@ int32 Insane::weaponMaxRange(int32 actornum) {
 
 	if (_actor[actornum].weapon == -1)
 		return 104;
-	
+
 	return map[_actor[actornum].weapon];
 }
 
@@ -883,7 +885,7 @@ int32 Insane::weaponMinRange(int32 actornum) {
 
 	if (_actor[actornum].weapon == -1)
 		return 40;
-	
+
 	return map[_actor[actornum].weapon];
 }
 
@@ -892,22 +894,28 @@ int32 Insane::weaponDamage(int32 actornum) {
 
 	if (_actor[actornum].weapon == -1)
 		return 10;
-	
+
 	return map[_actor[actornum].weapon];
 }
 
 void Insane::reinitActors(void) {
-	smlayer_setActorCostume(0, 2, readArray(12));
-	smlayer_setActorCostume(0, 0, readArray(14));
-	smlayer_setActorCostume(0, 1, readArray(13));
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+		smlayer_setActorCostume(0, 2, readArray(11));
+		smlayer_setActorCostume(0, 0, readArray(13));
+		smlayer_setActorCostume(0, 1, readArray(12));
+	} else {
+		smlayer_setActorCostume(0, 2, readArray(12));
+		smlayer_setActorCostume(0, 0, readArray(14));
+		smlayer_setActorCostume(0, 1, readArray(13));
+	}
 	smlayer_setActorLayer(0, 1, 1);
 	smlayer_setActorLayer(0, 2, 5);
 	smlayer_setActorLayer(0, 0, 10);
 	_actor[0].weapon = INV_HAND;
 	_actor[0].weaponClass = 2;
 	_actor[0].animWeaponClass = 0;
-	_actor[0].field_34 = 2;
-	_actor[0].field_38 = 0;
+	_actor[0].newFacingFlag = 2;
+	_actor[0].curFacingFlag = 0;
 	_actor[0].tilt = 0;
 	_actor[0].field_8 = 1;
 	_actor[0].act[2].state = 1;
@@ -924,7 +932,7 @@ int Insane::calcTilt(int speed) {
 	const int tilt[7] = {-5, -4, -2, 0, 2, 4, 5};
 	if (speed + 3 > 6)
 		return 0;
-	
+
 	return tilt[speed + 3];
 }
 
@@ -934,7 +942,7 @@ bool Insane::actor1StateFlags(int state) {
 	const int spans[] = {0, 34, 39, 73, 89, 90, 92, 93, 99, 100, 117};
 	bool retvalue = 0;
 	unsigned int i;
-	
+
 	for (i = 0; i < sizeof(spans); i++) {
 		retvalue = !retvalue;
 		if (spans[i] <= state)
@@ -955,28 +963,28 @@ void Insane::escapeKeyHandler(void) {
 	if (_needSceneSwitch || _keyboardDisable)
 		return;
 
-	debug(INSANE_DBG, "scene: %d", _currSceneId);
+	debugC(DEBUG_INSANE, "scene: %d", _currSceneId);
 	switch (_currSceneId) {
 	case 1:
 		queueSceneSwitch(1, _smush_minedrivFlu, "minedriv.san", 64, 0, _continueFrame1, 1300);
 		writeArray(9, 0);
 		break;
 	case 18:
-		queueSceneSwitch(1, _smush_minedrivFlu, "minedriv.san", 64, 0, _continueFrame1, 1300);
+		queueSceneSwitch(17, _smush_minedrivFlu, "minedriv.san", 64, 0, _continueFrame1, 1300);
 		writeArray(9, 1);
 		break;
 	case 2:
 		flu = &_fluConf[14 + _iactSceneId2];
-		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0, 
+		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0,
 						 flu->startFrame, flu->numFrames);
 		break;
 	case 3:
-		queueSceneSwitch(1, _smush_minedrivFlu, "minedriv.san", 64, 0, _continueFrame1, 1300);
+		queueSceneSwitch(1, _smush_minedrivFlu, "minedriv.san", 64, 0, _continueFrame, 1300);
 		break;
 	case 4:
 		if (_needSceneSwitch)
 			return;
-		
+
 		if (readArray(6)) {
 			if (readArray(4)) {
 				queueSceneSwitch(14, 0, "hitdust2.san", 64, 0, 0, 0);
@@ -1018,12 +1026,12 @@ void Insane::escapeKeyHandler(void) {
 		break;
 	case 8:
 		flu = &_fluConf[7 + _iactSceneId2];
-		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0, 
+		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0,
 						 flu->startFrame, flu->numFrames);
 		break;
 	case 7:
 		flu = &_fluConf[0 + _iactSceneId2];
-		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0, 
+		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0,
 						 flu->startFrame, flu->numFrames);
 		break;
 	case 23:
@@ -1072,7 +1080,7 @@ bool Insane::actor0StateFlags1(int state) {
 	const int spans[] = {0, 2, 34, 35, 39, 69, 98, 100, 117};
 	bool retvalue = 1;
 	unsigned int i;
-	
+
 	for (i = 0; i < sizeof(spans); i++) {
 		retvalue = !retvalue;
 		if (spans[i] >= state)
@@ -1092,7 +1100,7 @@ bool Insane::actor0StateFlags2(int state) {
 		 866, 867, 872, 920, 922, 923, 926, 927, 931, 933, 950};
 	bool retvalue = 1;
 	unsigned int i;
-	
+
 	for (i = 0; i < sizeof(spans); i++) {
 		retvalue = !retvalue;
 		if (spans[i] >= state)
@@ -1104,22 +1112,22 @@ bool Insane::actor0StateFlags2(int state) {
 // smlayer_loadSound1 && smlayer_loadSound2
 int Insane::smlayer_loadSound(int id, int flag, int phase) {
 	int resid;
-	
+
 	if (phase == 1) {
 		if (_idx2Exceeded != 0)
 			if (_objArray2Idx >= _objArray2Idx2)
 				return 0;
 	}
 	resid = readArray(id);
-	
+
 	if (!resid && phase == 2)
 		return 0;
-	
+
 	if (phase == 2)
 		_vm->ensureResourceLoaded(rtSound, resid);
-	
-	_vm->setResourceCounter(rtSound, resid, 1);
-	
+
+	_vm->res.setResourceCounter(rtSound, resid, 1);
+
 	if (phase == 1) {
 		_objArray2Idx2++;
 		_objArray2[_objArray2Idx2] = id;
@@ -1131,34 +1139,26 @@ int Insane::smlayer_loadSound(int id, int flag, int phase) {
 	return resid;
 }
 
-void Insane::IMUSE_shutVolume(void) {
-	_vm->_imuse->pause(true);
-}
-
-void Insane::IMUSE_restoreVolume(void) {
-	_vm->_imuse->pause(false);
-}
-
 // smlayer_loadCostume1 && smlayer_loadCostume2
 int Insane::smlayer_loadCostume(int id, int phase) {
 	int resid;
 	resid = readArray(id);
-	
+
 	if (!resid)
 		return 0;
-	
+
 	_vm->ensureResourceLoaded(rtCostume, resid);
-	_vm->setResourceCounter(rtCostume, resid, 1);
-	
+	_vm->res.setResourceCounter(rtCostume, resid, 1);
+
 	// smlayer_lock(rtCostume, resid); // FIXME
-	
+
 	if (phase == 1) {
 		_objArray1Idx2++;
 		_objArray1[_objArray1Idx2] = id;
 		if (_objArray1Idx2 == 100)
 			_objArray1Idx2 = 0;
 	}
-	
+
 	return resid;
 }
 
@@ -1177,10 +1177,13 @@ void Insane::smlayer_putActor(int actornum, int actnum, int x, int y, byte room)
 
 void Insane::smlayer_setActorLayer(int actornum, int actnum, int layer) {
 	Actor *a = _vm->derefActor(_actor[actornum].act[actnum].actor, "smlayer_setActorLayer");
-	a->layer = layer;
+	a->_layer = layer;
 }
 
 void Insane::smlayer_setFluPalette(byte *pal, int shut_flag) {
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+		return;
+
 	//	  if (shut_flag)
 	//		// FIXME: shut colors and make picture appear smoothly
 	//		SmushPlayer::setPalette(pal);
@@ -1216,8 +1219,8 @@ void Insane::smlayer_soundSetPriority(int32 soundId, int32 priority) {
 	_vm->_imuseDigital->setPriority(soundId, priority);
 }
 
-void Insane::smlayer_drawSomething(byte *renderBitmap, int32 codecparam, 
-			   int32 x, int32 y, int32 arg_10, NutRenderer *nutfile, 
+void Insane::smlayer_drawSomething(byte *renderBitmap, int32 codecparam,
+			   int32 x, int32 y, int32 arg_10, NutRenderer *nutfile,
 			   int32 c, int32 arg_1C, int32 arg_20) {
 	nutfile->drawFrame(renderBitmap, c, x, y);
 }
@@ -1228,8 +1231,8 @@ void Insane::smlayer_overrideDrawActorAt(byte *arg_0, byte arg_4, byte arg_8) {
 	// noop in current implementation
 }
 
-void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecparam, 
-					   int32 pos_x, int32 pos_y, int32 arg_14, int32 arg_18, 
+void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecparam,
+					   int32 pos_x, int32 pos_y, int32 arg_14, int32 arg_18,
 					   int32 flags, const char *formatString, const char *strng) {
 	SmushFont *sf = _player->_sf[0];
 	int color = 1;
@@ -1252,7 +1255,7 @@ void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecp
 			{
 				int id = str[3] - '0';
 				str += 4;
-				sf = _player->_sf[id]; 
+				sf = _player->_sf[id];
 			}
 			break;
 		case 'c':
@@ -1275,7 +1278,7 @@ void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecp
 	// bit 2 - ???          4
 	// bit 3 - wrap around  8
 	switch (flags) {
-	case 0: 
+	case 0:
 		sf->drawString(str, renderBitmap, _player->_width, _player->_height, pos_x, pos_y, false);
 		break;
 	case 1:
@@ -1285,19 +1288,28 @@ void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecp
 		sf->drawStringWrap(str, renderBitmap, _player->_width, _player->_height, pos_x, pos_y, 10, 300, true);
 		break;
 	default:
-		warning("Insane::smlayer_showStatusMsg. Not handled flags: %d", flags);
+		error("Insane::smlayer_showStatusMsg. Not handled flags: %d", flags);
 	}
 	free (string);
 }
 
 void Insane::procSKIP(Chunk &b) {
-	_player->checkBlock(b, TYPE_SKIP, 4);
 	int16 par1, par2;
+	_player->_skipNext = false;
+
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+		_player->checkBlock(b, TYPE_SKIP, 2);
+		par1 = b.getWord();
+		if (isBitSet(par1))
+			_player->_skipNext = true;
+		return;
+	}
+
+	_player->checkBlock(b, TYPE_SKIP, 4);
 
 	par1 = b.getWord();
 	par2 = b.getWord();
 
-	_player->_skipNext = false;
 
 	if (!par2) {
 		if (isBitSet(par1))
@@ -1339,18 +1351,18 @@ void Insane::smlayer_setActorFacing(int actornum, int actnum, int frame, int dir
 }
 
 const char *Insane::handleTrsTag(int32 trsId) {
-	debug(INSANE_DBG, "Insane::handleTrsTag(%d)", trsId);
+	debugC(DEBUG_INSANE, "Insane::handleTrsTag(%d)", trsId);
 	return _player->getString(trsId);
 }
 
 bool Insane::smush_eitherNotStartNewFrame(void) {
 	if (_smush_setupsan17)
 		return false;
-	
+
 	if (_smush_isSanFileSetup) {
 		if (_smush_frameStep < 0)
 			return false;
-	
+
 		if (_smush_curFrame > _smush_frameStep + _smush_frameNum2)
 			return true;
 		else
@@ -1358,7 +1370,7 @@ bool Insane::smush_eitherNotStartNewFrame(void) {
 	} else {
 		if (_smush_frameNum1 < 0)
 			return false;
-	
+
 		if (_smush_curFrame >= _smush_frameNum1) {
 			_smush_frameNum1 = -1;
 			return false;
@@ -1374,7 +1386,7 @@ int32 Insane::getLastKey(bool arg_0) {
 bool Insane::smlayer_actorNeedRedraw(int actornum, int actnum) {
 	Actor *a = _vm->derefActor(_actor[actornum].act[actnum].actor, "smlayer_actorNeedRedraw");
 
-	return a->needRedraw;
+	return a->_needRedraw;
 }
 
 int32 Insane::readArray (int item) {
@@ -1385,23 +1397,23 @@ void Insane::writeArray(int item, int value) {
 	_vm->writeArray(_numberArray, 0, item, value);
 }
 
-int32 Insane::smush_setupSanWithFlu(const char *filename, int32 setupsan2, int32 step1, 
-								   int32 step2, int32 setupsan1, byte *fluPtr, 
+int32 Insane::smush_setupSanWithFlu(const char *filename, int32 setupsan2, int32 step1,
+								   int32 step2, int32 setupsan1, byte *fluPtr,
 								   int32 numFrames) {
 	byte *tmp = fluPtr;
 	int32 offset;
-	
-	debug(INSANE_DBG, "smush_setupSanWithFlu(%s, %d, %d, %d, %d, %lx, %d)", filename, setupsan2,
+
+	debugC(DEBUG_INSANE, "smush_setupSanWithFlu(%s, %d, %d, %d, %d, %p, %d)", filename, setupsan2,
 		  step1, step2, setupsan1, fluPtr, numFrames);
 
 	_smush_setupsan1 = setupsan1;
-	
+
 	/* skip FLUP marker */
 	if (READ_BE_UINT32(fluPtr) == 'FLUP')
 		tmp += 8;
 
 	_smush_setupsan2 = setupsan2;
-	
+
 	if (tmp[2] <= 1) {
 		/* 0x300 -- palette, 0x8 -- header */
 		offset = READ_LE_UINT32(tmp + 0x308 + numFrames*4);
@@ -1427,12 +1439,12 @@ int32 Insane::smush_setupSanWithFlu(const char *filename, int32 setupsan2, int32
 	return offset;
 }
 
-void Insane::smush_setupSanFromStart(const char *filename, int32 setupsan2, int32 step1, 
+void Insane::smush_setupSanFromStart(const char *filename, int32 setupsan2, int32 step1,
 									 int32 step2, int32 setupsan1) {
-	debug(INSANE_DBG, "Insane::smush_setupFromStart(%s)", filename);
+	debugC(DEBUG_INSANE, "Insane::smush_setupFromStart(%s)", filename);
 	_smush_setupsan1 = setupsan1;
 	_smush_setupsan2 = setupsan2;
-	smush_setupSanFile(filename, 8, 0);
+	smush_setupSanFile(filename, 0, 0);
 	_smush_isSanFileSetup = 1;
 	smush_setFrameSteps(step1, step2);
 	smush_warpMouse(160, 100, -1);
@@ -1445,11 +1457,9 @@ void Insane::smush_setFrameSteps(int32 step1, int32 step2) {
 }
 
 void Insane::smush_setupSanFile(const char *filename, int32 offset, int32 contFrame) {
-	debug(INSANE_DBG, "Insane::smush_setupSanFile(%s, %x, %d)", filename, offset, contFrame);
+	debugC(DEBUG_INSANE, "Insane::smush_setupSanFile(%s, %x, %d)", filename, offset, contFrame);
 
-	_player->seekSan(filename, _vm->getGameDataPath(), offset, contFrame);
-
-	_vm->_imuseDigital->pause(false);
+	_player->seekSan(filename, offset, contFrame);
 }
 
 }
