@@ -1,6 +1,6 @@
 /* ScummVM - Scumm Interpreter
  * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -14,13 +14,13 @@
 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/scumm/insane/insane_ben.cpp,v 1.10.2.1 2004/06/23 03:24:55 sev Exp $
+ * $Header: /cvsroot/scummvm/scummvm/scumm/insane/insane_ben.cpp,v 1.30.2.2 2006/01/18 18:07:34 sev Exp $
  *
  */
 
-#include "stdafx.h"
+#include "common/stdafx.h"
 
 #include "base/engine.h"
 
@@ -33,7 +33,7 @@ int32 Insane::enemyBenHandler(int32 actor1, int32 actor2, int32 probability) {
 	int32 tmp;
 
 	retval = processMouse();
-	
+
 	// Joystick support is skipped
 
 	retval |= processKeyboard();
@@ -48,13 +48,13 @@ int32 Insane::enemyBenHandler(int32 actor1, int32 actor2, int32 probability) {
 	_actor[actor1].cursorX = tmp;
 
 	smush_warpMouse(_enemyState[EN_BEN][0], _enemyState[EN_BEN][1], -1);
-	
+
 	return (retval & 3);
 }
 
 void Insane::turnBen(bool controllable) {
 	int32 buttons;
-	
+
 	switch (_currSceneId) {
 	case 21:
 	case 25:
@@ -70,7 +70,7 @@ void Insane::turnBen(bool controllable) {
 				_actor[0].act[1].room = 0;
 				_actor[0].act[0].state = 36;
 				_actor[0].act[0].room = 0;
-		
+
 				if (smlayer_isSoundRunning(95))
 					smlayer_stopSound(95);
 			}
@@ -83,7 +83,7 @@ void Insane::turnBen(bool controllable) {
 			if (_currEnemy == EN_TORQUE)
 				buttons = 0;
 		}
-		debug(5, "00:%d 01:%d 02:%d 03:%d", _actor[0].act[0].state, 
+		debug(5, "00:%d 01:%d 02:%d 03:%d", _actor[0].act[0].state,
 				_actor[0].act[1].state, _actor[0].act[2].state, _actor[0].act[3].state);
 		actor01Reaction(buttons);
 		actor02Reaction(buttons);
@@ -122,6 +122,12 @@ void Insane::turnBen(bool controllable) {
 int32 Insane::actionBen(void) {
 	int32 buttons, tmp;
 	bool doDamage = false;
+	int sound;
+
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+		sound = 59;
+	else
+		sound = 95;
 
 	if (_actor[0].enemyHandler != -1)
 		buttons = enemyHandler(_actor[0].enemyHandler, 0, 1, _actor[0].probability);
@@ -165,12 +171,12 @@ int32 Insane::actionBen(void) {
 				_val213d = 0;
 			}
 
-			if (!smlayer_isSoundRunning(95))
-				smlayer_startSfx(95);
+			if (!smlayer_isSoundRunning(sound))
+				smlayer_startSfx(sound);
 		} else {
-			if (smlayer_isSoundRunning(95))
-				smlayer_stopSound(95);
-			
+			if (smlayer_isSoundRunning(sound))
+				smlayer_stopSound(sound);
+
 			_val213d = 0;
 		}
 	} else {
@@ -211,15 +217,6 @@ int32 Insane::processBenOnRoad(bool flag) {
 		if (_actor[0].speed < -8)
 			_actor[0].speed = -8;
 
-		// FIXME: is it abs(/2) ?
-		// mov   eax, insane_actor0.speed
-		// mov   edx, eax
-		// sar   edx, 1Fh
-		// sub   eax, edx
-		// sar   eax, 1
-		// add   eax, insane_actor0.speed
-		// add   insane_actor0.x, eax
-
 		_actor[0].x += _actor[0].speed / 2 + _actor[0].speed;
 
 		if (_actor[0].x < 0)
@@ -259,7 +256,7 @@ void Insane::mineChooseRoad(int32 buttons) {
 					queueSceneSwitch(2, 0, "mineexit.san", 64, 0, 0, 0);
 				}
 			}
-			
+
 			if ((buttons & 2) == 0 || _needSceneSwitch)
 				return;
 
@@ -295,9 +292,9 @@ void Insane::mineChooseRoad(int32 buttons) {
 				_actor[0].tilt = -7;
 			if (tmp > 7)
 				_actor[0].tilt = 7;
-			
+
 			drawSpeedyActor(buttons);
-			
+
 			if ((buttons & 1) == 0)
 				return;
 
@@ -311,7 +308,7 @@ void Insane::mineChooseRoad(int32 buttons) {
 					queueSceneSwitch(8, 0, "tomine.san", 64, 0, 0, 0);
 				}
 			}
-			
+
 			if (_roadStop) {
 				writeArray(1, _posBrokenTruck);
 				writeArray(3, _val57d);
@@ -334,7 +331,7 @@ void Insane::mineChooseRoad(int32 buttons) {
 				_actor[0].tilt = 7;
 
 			drawSpeedyActor(buttons);
-			
+
 			if ((buttons & 1) == 0)
 				return;
 
@@ -348,7 +345,7 @@ void Insane::mineChooseRoad(int32 buttons) {
 					queueSceneSwitch(7, 0, "tomine.san", 64, 0, 0, 0);
 				}
 			}
-			
+
 			if (_roadStop) {
 				writeArray(1, _posBrokenTruck);
 				writeArray(3, _posVista);
@@ -502,7 +499,7 @@ int32 Insane::calcBenDamage(bool arg_0, bool arg_4) {
 // Ben
 void Insane::actor02Reaction(int32 buttons) {
 	int32 tmp, tmp2;
-	
+
 	switch(_actor[0].act[2].state) {
 	case 1:
 		smlayer_setActorLayer(0, 2, 5);
@@ -568,10 +565,15 @@ void Insane::actor02Reaction(int32 buttons) {
 		if (_actor[0].act[2].frame == 2) {
 			if (_currEnemy != EN_CAVEFISH) {
 				tmp = calcEnemyDamage(1, 1);
-				if (tmp == 1)
-					smlayer_startSfx(60);
-				if (tmp == 1000)
-					smlayer_startSfx(62);
+				if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+					if (tmp == 1)
+						smlayer_startSfx(50);
+				} else {
+					if (tmp == 1)
+						smlayer_startSfx(60);
+					if (tmp == 1000)
+						smlayer_startSfx(62);
+				}
 			} else {
 				if ((_actor[1].x - _actor[0].x <= weaponMaxRange(0)) &&
 					(_actor[1].x - _actor[0].x >= weaponMinRange(0)) &&
@@ -583,7 +585,7 @@ void Insane::actor02Reaction(int32 buttons) {
 			smlayer_setActorFacing(0, 2, 20, 180);
 			_actor[0].act[2].state = 4;
 		}
-	
+
 		_actor[0].kicking = true;
 		_actor[0].act[2].tilt = calcTilt(_actor[0].tilt);
 		break;
@@ -605,7 +607,7 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 6:
 		smlayer_setActorLayer(0, 2, 4);
 		_actor[0].weaponClass = 2;
-		_actor[0].field_34 = 1;
+		_actor[0].newFacingFlag = 1;
 		_actor[0].kicking = false;
 		smlayer_setActorCostume(0, 2, readArray(22));
 		smlayer_setActorFacing(0, 2, 19, 180);
@@ -616,7 +618,7 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 7:
 		smlayer_setActorLayer(0, 2, 4);
 		_actor[0].weaponClass = 2;
-		_actor[0].field_34 = 1;
+		_actor[0].newFacingFlag = 1;
 		_actor[0].kicking = false;
 		if (_actor[0].act[2].frame >= 1) {
 			smlayer_setActorFacing(0, 2, 20, 180);
@@ -627,7 +629,7 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 8:
 		smlayer_setActorLayer(0, 2, 4);
 		_actor[0].weaponClass = 2;
-		_actor[0].field_34 = 1;
+		_actor[0].newFacingFlag = 1;
 		_actor[0].kicking = false;
 		if ((_actor[0].act[2].frame == 3) && (calcEnemyDamage(0, 0) == 1)) {
 			_actor[1].damage = weaponDamage(0);
@@ -643,11 +645,11 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 9:
 		smlayer_setActorLayer(0, 2, 5);
 		_actor[0].weaponClass = 2;
-		_actor[0].field_34 = 1;
+		_actor[0].newFacingFlag = 1;
 		_actor[0].kicking = false;
 		if (_actor[0].act[2].frame >= 3) {
 			smlayer_setActorCostume(0, 2, readArray(12));
-			_actor[0].field_34 = 2;
+			_actor[0].newFacingFlag = 2;
 			_actor[0].act[2].state = 1;
 		}
 		_actor[0].act[2].tilt = calcTilt(_actor[0].tilt);
@@ -714,7 +716,7 @@ void Insane::actor02Reaction(int32 buttons) {
 						smlayer_startSfx(74);
 					break;
 				default:
-					if(calcEnemyDamage(1, 0) == 1)
+					if (calcEnemyDamage(1, 0) == 1)
 						smlayer_startSfx(73);
 					break;
 				}
@@ -723,7 +725,7 @@ void Insane::actor02Reaction(int32 buttons) {
 					(_actor[1].x - _actor[0].x >= weaponMinRange(0)) &&
 					!_actor[0].field_54)
 					prepareScenePropScene(1, 0, 0);
-		
+
 			}
 			smlayer_setActorFacing(0, 2, 21, 180);
 			_actor[0].act[2].state = 13;
@@ -764,7 +766,7 @@ void Insane::actor02Reaction(int32 buttons) {
 					smlayer_setActorFacing(0, 2, 20, 180);
 					_actor[0].act[2].state = 16;
 				}
-				break;	
+				break;
 			case INV_MACE:
 				if (!_actor[1].kicking || _actor[1].field_44)
 					if (actor1StateFlags(_actor[1].act[2].state)) {
@@ -886,10 +888,17 @@ void Insane::actor02Reaction(int32 buttons) {
 				case INV_2X4:
 				case INV_BOOT:
 					tmp = calcEnemyDamage(1, 1);
-					if (tmp == 1)
-						smlayer_startSfx(67);
-					if (tmp == 1000)
-						smlayer_startSfx(68);
+					if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+						if (tmp == 1)
+							smlayer_startSfx(52);
+						if (tmp == 1000)
+							smlayer_startSfx(56);
+					} else {
+						if (tmp == 1)
+							smlayer_startSfx(67);
+						if (tmp == 1000)
+							smlayer_startSfx(68);
+					}
 					break;
 				default:
 					if (calcEnemyDamage(1, 0))
@@ -1004,7 +1013,8 @@ void Insane::actor02Reaction(int32 buttons) {
 		smlayer_setActorFacing(0, 2, 19, 180);
 		_actor[0].act[2].state = 27;
 		_actor[0].act[2].tilt = calcTilt(_actor[0].tilt);
-		smlayer_startSfx(72);
+		if (!((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)))
+			smlayer_startSfx(72);
 		break;
 	case 27:
 		smlayer_setActorLayer(0, 2, 4);
@@ -1046,10 +1056,17 @@ void Insane::actor02Reaction(int32 buttons) {
 				case INV_BOOT:
 				case INV_DUST:
 					tmp = calcEnemyDamage(1, 1);
-					if (tmp == 1)
-						smlayer_startSfx(70);
-					if (tmp == 1000)
-						smlayer_startSfx(71);
+					if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+						if (tmp == 1)
+							smlayer_startSfx(58);
+						if (tmp == 1000)
+							smlayer_startSfx(56);
+					} else {
+						if (tmp == 1)
+							smlayer_startSfx(70);
+						if (tmp == 1000)
+							smlayer_startSfx(71);
+					}
 					break;
 				case INV_HAND:
 					if (!calcEnemyDamage(1, 0))
@@ -1136,7 +1153,7 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 34:
 		smlayer_setActorLayer(0, 2, 5);
 		_actor[0].kicking = false;
-		
+
 		if (!smlayer_actorNeedRedraw(0, 2)) {
 			setBenState();
 			_actor[0].act[2].tilt = 0;
@@ -1148,7 +1165,7 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 35:
 		smlayer_setActorLayer(0, 2, 5);
 		_actor[0].kicking = false;
-		
+
 		if (!smlayer_actorNeedRedraw(0, 2)) {
 			switchBenWeapon();
 			_actor[0].act[2].tilt = 0;
@@ -1158,7 +1175,10 @@ void Insane::actor02Reaction(int32 buttons) {
 	case 36:
 		smlayer_setActorLayer(0, 2, 5);
 		_actor[0].kicking = false;
-		smlayer_setActorCostume(0, 2, readArray(18));
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+			smlayer_setActorCostume(0, 2, readArray(17));
+		else
+			smlayer_setActorCostume(0, 2, readArray(18));
 		smlayer_setActorFacing(0, 2, 6, 180);
 		smlayer_startSfx(96);
 		switch (_currEnemy) {
@@ -1191,7 +1211,7 @@ void Insane::actor02Reaction(int32 buttons) {
 		smlayer_setActorLayer(0, 2, 25);
 		_actor[0].cursorX = 0;
 		_actor[0].kicking = false;
-		if (_actor[0].act[2].frame >= 18 || 
+		if (_actor[0].act[2].frame >= 18 ||
 			(_actor[0].x < 50 && _actor[0].act[2].frame >= 10) ||
 			 (_actor[0].x > 270 && _actor[0].act[2].frame >= 10)) {
 			if (_currSceneId == 21) {
@@ -1201,7 +1221,10 @@ void Insane::actor02Reaction(int32 buttons) {
 				case EN_ROTT1:
 				case EN_ROTT2:
 				case EN_ROTT3:
-					queueSceneSwitch(9, 0, "wr2_benr.san", 64, 0, 0, 0);
+					if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+						queueSceneSwitch(9, 0, "bencrshe.san", 64, 0, 0, 0);
+					else
+						queueSceneSwitch(9, 0, "wr2_benr.san", 64, 0, 0, 0);
 					break;
 				case EN_VULTF1:
 				case EN_VULTM1:
@@ -1608,7 +1631,7 @@ void Insane::actor01Reaction(int32 buttons) {
 			setBenAnimation(0, 6);
 			_actor[0].act[1].state = 41;
 		}
-	
+
 		if (_actor[0].cursorX >= -100) {
 			setBenAnimation(0, 7);
 			_actor[0].act[1].state = 40;
@@ -1631,7 +1654,7 @@ void Insane::actor01Reaction(int32 buttons) {
 			setBenAnimation(0, 8);
 			_actor[0].act[1].state = 39;
 		}
-	
+
 		if (_actor[0].field_8 == 48)
 			_actor[0].tilt = 0;
 		else
@@ -1682,7 +1705,7 @@ void Insane::actor01Reaction(int32 buttons) {
 			setBenAnimation(0, 12);
 			_actor[0].act[1].state = 57;
 		}
-	
+
 		if (_actor[0].cursorX <= 100) {
 			setBenAnimation(0, 11);
 			_actor[0].act[1].state = 56;
@@ -1691,9 +1714,9 @@ void Insane::actor01Reaction(int32 buttons) {
 		}
 		break;
 	}
-	
-	if (_actor[0].field_38 != _actor[0].field_34) {
-		if (_actor[0].field_34 == 2)
+
+	if (_actor[0].curFacingFlag != _actor[0].newFacingFlag) {
+		if (_actor[0].newFacingFlag == 2)
 			smlayer_setActorFacing(0, 1, 28, 180);
 		else
 			smlayer_setActorFacing(0, 1, 27, 180);
@@ -1704,11 +1727,11 @@ void Insane::actor01Reaction(int32 buttons) {
 
 	if (_actor[0].act[1].room)
 		smlayer_putActor(0, 1, tmpx, tmpy, _smlayer_room2);
-	else 
+	else
 		smlayer_putActor(0, 1, tmpx, tmpy, _smlayer_room);
 
 	_actor[0].animWeaponClass = _actor[0].weaponClass;
-	_actor[0].field_38 = _actor[0].field_34;
+	_actor[0].curFacingFlag = _actor[0].newFacingFlag;
 }
 
 void Insane::actor03Reaction(int32 buttons) {
@@ -1721,7 +1744,7 @@ void Insane::actor03Reaction(int32 buttons) {
 	case 52:
 		if (_actor[0].runningSound)
 			smlayer_stopSound(_actor[0].runningSound);
-		
+
 		if (_currScenePropIdx)
 			shutCurrentScene();
 
@@ -1854,7 +1877,7 @@ void Insane::chooseBenWeaponAnim(int buttons) {
 			_actor[0].act[2].state = 0;
 			switchBenWeapon();
 		}
-		
+
 		_weaponBenJustSwitched = true;
 	} else {
 		_weaponBenJustSwitched = false;
@@ -1873,19 +1896,19 @@ void Insane::switchBenWeapon(void) {
 	case INV_CHAIN:
 		smlayer_setActorCostume(0, 2, readArray(20));
 		smlayer_setActorFacing(0, 2, 18, 180);
-		_actor[0].weaponClass = 0;
+		_actor[0].weaponClass = 1;
 		_actor[0].act[2].state = 34;
 		break;
 	case INV_CHAINSAW:
 		smlayer_setActorCostume(0, 2, readArray(24));
 		smlayer_setActorFacing(0, 2, 18, 180);
-		_actor[0].weaponClass = 0;
+		_actor[0].weaponClass = 1;
 		_actor[0].act[2].state = 34;
 		break;
 	case INV_MACE:
 		smlayer_setActorCostume(0, 2, readArray(23));
 		smlayer_setActorFacing(0, 2, 18, 180);
-		_actor[0].weaponClass = 0;
+		_actor[0].weaponClass = 1;
 		_actor[0].act[2].state = 34;
 		break;
 	case INV_2X4:
@@ -1895,19 +1918,25 @@ void Insane::switchBenWeapon(void) {
 			smlayer_setActorCostume(0, 2, readArray(19));
 
 		smlayer_setActorFacing(0, 2, 18, 180);
-		_actor[0].weaponClass = 0;
+		_actor[0].weaponClass = 1;
 		_actor[0].act[2].state = 34;
 		break;
 	case INV_WRENCH:
-		smlayer_setActorCostume(0, 2, readArray(25));
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+			smlayer_setActorCostume(0, 2, readArray(24));
+		else
+			smlayer_setActorCostume(0, 2, readArray(25));
 		smlayer_setActorFacing(0, 2, 18, 180);
-		_actor[0].weaponClass = 0;
+		_actor[0].weaponClass = 1;
 		_actor[0].act[2].state = 34;
 		break;
 	case INV_BOOT:
 	case INV_HAND:
 	case INV_DUST:
-		smlayer_setActorCostume(0, 2, readArray(12));
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+			smlayer_setActorCostume(0, 2, readArray(11));
+		else
+			smlayer_setActorCostume(0, 2, readArray(12));
 		_actor[0].weaponClass = 2;
 		_actor[0].act[2].state = 1;
 		break;
@@ -1955,6 +1984,11 @@ int32 Insane::setBenState(void) {
 void Insane::ouchSoundBen(void) {
 	_actor[0].act[3].state = 52;
 
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+		smlayer_startVoice(54);
+		return;
+	}
+
 	switch (_vm->_rnd.getRandomNumber(3)) {
 	case 0:
 		smlayer_startVoice(315);
