@@ -1,6 +1,6 @@
 /* ScummVM - Scumm Interpreter
  * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2004 The ScummVM project
+ * Copyright (C) 2001-2006 The ScummVM project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -14,13 +14,13 @@
 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/scumm/insane/insane_scenes.cpp,v 1.14 2004/02/09 18:57:03 sev Exp $
+ * $Header: /cvsroot/scummvm/scummvm/scumm/insane/insane_scenes.cpp,v 1.35.2.2 2006/01/18 18:07:34 sev Exp $
  *
  */
 
-#include "stdafx.h"
+#include "common/stdafx.h"
 
 #include "base/engine.h"
 
@@ -34,11 +34,6 @@
 namespace Scumm {
 
 void Insane::runScene(int arraynum) {
-#ifndef FTDOSDEMO
-	if ((_vm->_features & GF_DEMO) && (_vm->_features & GF_PC))
-		return;
-#endif
-
 	_insaneIsRunning = true;
 	_player = new SmushPlayer(_vm, _speed);
 	_player->insanity(true);
@@ -60,7 +55,7 @@ void Insane::runScene(int arraynum) {
 	putActors();
 	readState();
 
-	debug(INSANE_DBG, "INSANE Arg: %d", readArray(0));
+	debugC(DEBUG_INSANE, "INSANE Arg: %d", readArray(0));
 
 	switch (readArray(0)) {
 	case 1:
@@ -72,11 +67,17 @@ void Insane::runScene(int arraynum) {
 		break;
 	case 2:
 		setupValues();
-		smlayer_setActorCostume(0, 2, readArray(11));
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+			smlayer_setActorCostume(0, 2, readArray(10));
+		else
+			smlayer_setActorCostume(0, 2, readArray(11));
 		smlayer_putActor(0, 2, _actor[0].x, _actor[0].y1 + 190, _smlayer_room2);
 
 		_mainRoadPos = readArray(2);
-		if (_mainRoadPos == _posBrokenTruck) {
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+			initScene(5);
+			startVideo("tovista.san", 1, 32, 12, 0);
+		} else if (_mainRoadPos == _posBrokenTruck) {
 			initScene(5);
 			startVideo("tovista2.san", 1, 32, 12, 0);
 		} else if (_mainRoadPos == _posBrokenCar) {
@@ -130,46 +131,44 @@ void Insane::runScene(int arraynum) {
 		initScene(26);
 		writeArray(1, _val54d);
 		startVideo("credits.san", 1, 32, 12, 0);
-	default:
-		warning("Unknown FT_INSANE mode %d", readArray(0));
 		break;
+	default:
+		error("Unknown FT_INSANE mode %d", readArray(0));
 	}
 
 	putActors();
-	_vm->_sound->pauseSounds(0); // IMUSE_Resume();
 	_enemy[EN_ROTT3].maxdamage = 120;
 
 	_insaneIsRunning = false;
 
-	writeArray(50, _actor[0].inventory[INV_CHAIN]);
-	writeArray(51, _actor[0].inventory[INV_CHAINSAW]);
-	writeArray(52, _actor[0].inventory[INV_MACE]);
-	writeArray(53, _actor[0].inventory[INV_2X4]);
-	writeArray(54, _actor[0].inventory[INV_WRENCH]);
-	writeArray(55, _actor[0].inventory[INV_DUST]);
-	writeArray(337, _enemy[EN_TORQUE].occurences);
-	writeArray(329, _enemy[EN_ROTT1].occurences);
-	writeArray(330, _enemy[EN_ROTT2].occurences);
-	writeArray(331, _enemy[EN_ROTT3].occurences);
-	writeArray(332, _enemy[EN_VULTF1].occurences);
-	writeArray(333, _enemy[EN_VULTM1].occurences);
-	writeArray(334, _enemy[EN_VULTF2].occurences);
-	writeArray(335, _enemy[EN_VULTM2].occurences);
-	writeArray(336, _enemy[EN_CAVEFISH].occurences);
-	writeArray(339, _enemy[EN_VULTF2].field_10);
-	writeArray(56, _enemy[EN_CAVEFISH].field_10);
-	writeArray(340, _enemy[EN_VULTM2].field_10);
+	if (!((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))) {
+		writeArray(50, _actor[0].inventory[INV_CHAIN]);
+		writeArray(51, _actor[0].inventory[INV_CHAINSAW]);
+		writeArray(52, _actor[0].inventory[INV_MACE]);
+		writeArray(53, _actor[0].inventory[INV_2X4]);
+		writeArray(54, _actor[0].inventory[INV_WRENCH]);
+		writeArray(55, _actor[0].inventory[INV_DUST]);
+		writeArray(56, _enemy[EN_CAVEFISH].isEmpty);
+		writeArray(337, _enemy[EN_TORQUE].occurences);
+		writeArray(329, _enemy[EN_ROTT1].occurences);
+		writeArray(330, _enemy[EN_ROTT2].occurences);
+		writeArray(331, _enemy[EN_ROTT3].occurences);
+		writeArray(332, _enemy[EN_VULTF1].occurences);
+		writeArray(333, _enemy[EN_VULTM1].occurences);
+		writeArray(334, _enemy[EN_VULTF2].occurences);
+		writeArray(335, _enemy[EN_VULTM2].occurences);
+		writeArray(336, _enemy[EN_CAVEFISH].occurences);
+		writeArray(339, _enemy[EN_VULTF2].isEmpty);
+		writeArray(340, _enemy[EN_VULTM2].isEmpty);
+	}
 	// insane_unlock(); // FIXME
 	_vm->_sound->stopAllSounds(); // IMUSE_StopAllSounds();
-	if (_memoryAllocatedNotOK) {
-		error("Memory Alloc Error in Mineroad (Heap Size?)");
-	}
-	
+
 	delete _player;
 }
 
 int Insane::initScene(int sceneId) {
-	debug(INSANE_DBG, "initScene(%d)", sceneId);
+	debugC(DEBUG_INSANE, "initScene(%d)", sceneId);
 
 	if (_needSceneSwitch)
 		return 1;
@@ -191,7 +190,7 @@ int Insane::initScene(int sceneId) {
 void Insane::stopSceneSounds(int sceneId) {
 	int flag = 0;
 
-	debug(INSANE_DBG, "stopSceneSounds(%d)", sceneId);
+	debugC(DEBUG_INSANE, "stopSceneSounds(%d)", sceneId);
 
 	switch (sceneId) {
 	case 1:
@@ -223,14 +222,14 @@ void Insane::stopSceneSounds(int sceneId) {
 		if (_actor[0].runningSound != 0)
 			smlayer_stopSound(_actor[0].runningSound);
 		_actor[0].runningSound = 0;
-	
+
 		if (_actor[1].runningSound != 0)
 			smlayer_stopSound(_actor[1].runningSound);
 		_actor[1].runningSound = 0;
-	
+
 		if (_currScenePropIdx != 0)
 			shutCurrentScene();
-	
+
 		_currScenePropSubIdx = 0;
 		_currTrsMsg = 0;
 		_actor[0].defunct = 0;
@@ -239,13 +238,18 @@ void Insane::stopSceneSounds(int sceneId) {
 		_actor[1].defunct = 0;
 		_actor[1].scenePropSubIdx = 0;
 		_actor[1].field_54 = 0;
-		smlayer_stopSound(89);
-		smlayer_stopSound(90);
-		smlayer_stopSound(91);
-		smlayer_stopSound(92);
-		smlayer_stopSound(93);
-		smlayer_stopSound(95);
-		smlayer_stopSound(87);
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+			smlayer_stopSound(59);
+			smlayer_stopSound(63);
+		} else {
+			smlayer_stopSound(89);
+			smlayer_stopSound(90);
+			smlayer_stopSound(91);
+			smlayer_stopSound(92);
+			smlayer_stopSound(93);
+			smlayer_stopSound(95);
+			smlayer_stopSound(87);
+		}
 		break;
 	case 4:
 	case 5:
@@ -278,19 +282,19 @@ void Insane::stopSceneSounds(int sceneId) {
 	smlayer_setActorCostume(1, 2, 0);
 	smlayer_setActorCostume(1, 0, 0);
 	smlayer_setActorCostume(1, 1, 0);
-	
+
 	return;
 }
 
 void Insane::shutCurrentScene(void) {
-	debug(INSANE_DBG, "shutCurrentScene()");
+	debugC(DEBUG_INSANE, "shutCurrentScene()");
 
 	_currScenePropIdx = 0;
 	_currTrsMsg = 0;
 	_currScenePropSubIdx = 0;
 	_actor[1].scenePropSubIdx = 0;
 	_actor[1].defunct = 0;
-	
+
 	if (_actor[1].runningSound != 0) {
 		smlayer_stopSound(_actor[1].runningSound);
 		_actor[1].runningSound = 0;
@@ -310,9 +314,12 @@ void Insane::shutCurrentScene(void) {
 
 // insane_loadSceneData1 & insane_loadSceneData2
 int Insane::loadSceneData(int scene, int flag, int phase) {
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+		return 1;
+
 	int retvalue = 1;
 
-	debug(INSANE_DBG, "Insane::loadSceneData(%d, %d, %d)", scene, flag, phase);
+	debugC(DEBUG_INSANE, "Insane::loadSceneData(%d, %d, %d)", scene, flag, phase);
 	//if (phase == 1) /// FIXME
 	//	insane_unlock();
 	switch (scene) {
@@ -606,7 +613,7 @@ int Insane::loadSceneData(int scene, int flag, int phase) {
 }
 
 void Insane::setSceneCostumes(int sceneId) {
-	debug(INSANE_DBG, "Insane::setSceneCostumes(%d)", sceneId);
+	debugC(DEBUG_INSANE, "Insane::setSceneCostumes(%d)", sceneId);
 
 	switch (sceneId) {
 	case 1:
@@ -641,7 +648,10 @@ void Insane::setSceneCostumes(int sceneId) {
 	case 4:
 	case 5:
 	case 6:
-		smlayer_setActorCostume(0, 2, readArray(11));
+		if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+			smlayer_setActorCostume(0, 2, readArray(10));
+		else
+			smlayer_setActorCostume(0, 2, readArray(11));
 		smlayer_putActor(0, 2, _actor[0].x, _actor[0].y1+190, _smlayer_room2);
 		setupValues();
 		return;
@@ -656,18 +666,24 @@ void Insane::setSceneCostumes(int sceneId) {
 void Insane::setEnemyCostumes(void) {
 	int i;
 
-	debug(INSANE_DBG, "setEnemyCostumes(%d)", _currEnemy);
+	debugC(DEBUG_INSANE, "setEnemyCostumes(%d)", _currEnemy);
 
-	smlayer_setActorCostume(0, 2, readArray(12));
-	smlayer_setActorCostume(0, 0, readArray(14));
-	smlayer_setActorCostume(0, 1, readArray(13));
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC)) {
+		smlayer_setActorCostume(0, 2, readArray(11));
+		smlayer_setActorCostume(0, 0, readArray(13));
+		smlayer_setActorCostume(0, 1, readArray(12));
+	} else {
+		smlayer_setActorCostume(0, 2, readArray(12));
+		smlayer_setActorCostume(0, 0, readArray(14));
+		smlayer_setActorCostume(0, 1, readArray(13));
+	}
 	smlayer_setActorLayer(0, 1, 1);
 	smlayer_setActorLayer(0, 2, 5);
 	smlayer_setActorLayer(0, 0, 10);
 	smlayer_putActor(0, 2, _actor[0].x+11, _actor[0].y1+102, _smlayer_room2);
 	smlayer_putActor(0, 1, _actor[0].x, _actor[0].y1+200, _smlayer_room2);
 	smlayer_putActor(0, 0, _actor[0].x, _actor[0].y1+200, _smlayer_room2);
-	
+
 	if (_currEnemy == EN_CAVEFISH) {
 		smlayer_setActorCostume(1, 2, readArray(_enemy[_currEnemy].costume4));
 		_actor[1].act[2].room = 1;
@@ -678,8 +694,8 @@ void Insane::setEnemyCostumes(void) {
 		_actor[1].act[2].state = 98;
 		_actor[1].act[0].state = 98;
 		_actor[1].act[1].state = 98;
-	
-		smlayer_putActor(1, 2, _actor[1].x + _actor[1].act[2].tilt - 17, 
+
+		smlayer_putActor(1, 2, _actor[1].x + _actor[1].act[2].tilt - 17,
 						 _actor[1].y + _actor[1].y1 - 98, _smlayer_room2);
 	} else if (_currEnemy == EN_TORQUE) {
 		smlayer_setActorCostume(1, 2, readArray(_enemy[_currEnemy].costume4));
@@ -690,45 +706,45 @@ void Insane::setEnemyCostumes(void) {
 		_actor[1].act[2].state = 1;
 		_actor[1].act[0].state = 1;
 		_actor[1].act[1].state = 1;
-		smlayer_putActor(1, 2, _actor[1].x + _actor[1].act[2].tilt - 17, 
+		smlayer_putActor(1, 2, _actor[1].x + _actor[1].act[2].tilt - 17,
 						 _actor[1].y + _actor[1].y1 - 98, _smlayer_room2);
 	} else {
 		_actor[1].act[2].room = 1;
 		_actor[1].act[1].room = 1;
 		_actor[1].act[0].room = 1;
-	
+
 		if (_enemy[_currEnemy].costume4)
 			smlayer_setActorCostume(1, 2, readArray(_enemy[_currEnemy].costume4));
-	
+
 		if (_enemy[_currEnemy].costume5)
 			smlayer_setActorCostume(1, 0, readArray(_enemy[_currEnemy].costume5));
-	
+
 		if (_enemy[_currEnemy].costume6)
 			smlayer_setActorCostume(1, 1, readArray(_enemy[_currEnemy].costume6));
-	
+
 		_actor[1].field_8 = 1;
 		_actor[1].act[2].state = 1;
 		_actor[1].act[0].state = 1;
 		_actor[1].act[1].state = 1;
-	
+
 		if (_actor[1].act[2].room != 0)
-			smlayer_putActor(1, 2, _actor[1].x + _actor[1].act[2].tilt - 17, 
-							 _actor[1].y + _actor[1].y1 - 98, 
+			smlayer_putActor(1, 2, _actor[1].x + _actor[1].act[2].tilt - 17,
+							 _actor[1].y + _actor[1].y1 - 98,
 							 _smlayer_room2);
 	}
-	
+
 	if (_actor[1].act[1].room != 0)
 		smlayer_putActor(1, 1, _actor[1].x, _actor[1].y + _actor[1].y1,
 						 _smlayer_room2);
-	
+
 	if (_actor[1].act[0].room != 0)
-		smlayer_putActor(1, 0, _actor[1].x, _actor[1].y + _actor[1].y1, 
+		smlayer_putActor(1, 0, _actor[1].x, _actor[1].y + _actor[1].y1,
 						 _smlayer_room2);
-	
+
 	smlayer_setActorLayer(1, 1, 1);
 	smlayer_setActorLayer(1, 2, 5);
 	smlayer_setActorLayer(1, 0, 10);
-	
+
 	_actor[1].damage = 0;
 	_actor[1].x = 250;
 	_actor[1].y = 300;
@@ -748,8 +764,8 @@ void Insane::setEnemyCostumes(void) {
 	_actor[0].weapon = INV_HAND;
 	_actor[0].weaponClass = 2;
 	_actor[0].animWeaponClass = 0;
-	_actor[0].field_34 = 2;
-	_actor[0].field_38 = 0;
+	_actor[0].newFacingFlag = 2;
+	_actor[0].curFacingFlag = 0;
 	_actor[0].tilt = 0;
 	_actor[0].field_8 = 1;
 	_actor[0].act[2].state = 1;
@@ -778,7 +794,7 @@ void Insane::setEnemyCostumes(void) {
 	_actor[1].field_44 = false;
 	_actor[1].field_48 = false;
 	if (_enemy[_currEnemy].initializer != -1)
-		enemyInitializer(_enemy[_currEnemy].initializer, _actor[1].damage, 
+		enemyInitializer(_enemy[_currEnemy].initializer, _actor[1].damage,
 							 _actor[0].damage, _actor[1].probability);
 
 	smush_warpMouse(160, 100, -1);
@@ -793,17 +809,12 @@ void Insane::procPreRendering(void) {
 		_val115_ = true;
 		if (!_keyboardDisable) {
 			smush_changeState(1);
-			_smush_isPauseImuse = true;
-			IMUSE_shutVolume();
 			_keyboardDisable = 1;
 		}
 	} else {
 		_val115_ = false;
 		if (_keyboardDisable) {
 			smush_changeState(0);
-			_smush_isPauseImuse = false;
-			IMUSE_restoreVolume();
-
 			_keyboardDisable = 0;
 		}
 	}
@@ -816,17 +827,17 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 	int32 tmpSnd;
 	bool needMore = false;
 
-	if(!_keyboardDisable) {
+	if (!_keyboardDisable) {
 		switch (_currSceneId) {
 		case 12:
 			postCase11(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
 			break;
 		case 1:
 			postCase0(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
-			if(!smlayer_isSoundRunning(88))
+			if (!smlayer_isSoundRunning(88))
 				smlayer_startSfx(88);
 			smlayer_soundSetPan(88, ((_actor[0].x+160)>>2)+64);
-			if(_tiresRustle) {
+			if (_tiresRustle) {
 				if (!smlayer_isSoundRunning(87))
 					smlayer_startSfx(87);
 			} else {
@@ -839,12 +850,12 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 			smlayer_stopSound(95);
 			smlayer_stopSound(87);
 			smlayer_stopSound(88);
-			if(!smlayer_isSoundRunning(88))
+			if (!smlayer_isSoundRunning(88))
 				smlayer_startSfx(88);
 			break;
 		case 17:
 			postCase16(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
-			if(!smlayer_isSoundRunning(88))
+			if (!smlayer_isSoundRunning(88))
 				smlayer_startSfx(88);
 			break;
 		case 2:
@@ -853,18 +864,18 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 		case 3:
 			postCase2(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
 			needMore = true;
-			if(!smlayer_isSoundRunning(89)) {
+			if (!smlayer_isSoundRunning(89)) {
 				smlayer_startSfx(89);
 				smlayer_soundSetPriority(89, 100);
 			}
 			tmpSnd = _enemy[_currEnemy].sound;
-			if(!smlayer_isSoundRunning(tmpSnd)) {
+			if (!smlayer_isSoundRunning(tmpSnd)) {
 				smlayer_startSfx(tmpSnd);
 				smlayer_soundSetPriority(tmpSnd, 100);
 			}
 			smlayer_soundSetPan(89, ((_actor[0].x+160)>>2)+64);
 			smlayer_soundSetPan(tmpSnd, ((_actor[1].x+160)>>2)+64);
-			if(!_tiresRustle) {
+			if (!_tiresRustle) {
 				smlayer_stopSound(87);
 			} else {
 				if (!smlayer_isSoundRunning(87))
@@ -874,12 +885,12 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 		case 21:
 			postCase20(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
 			needMore = true;
-			if(!smlayer_isSoundRunning(89)) {
+			if (!smlayer_isSoundRunning(89)) {
 				smlayer_startSfx(89);
 				smlayer_soundSetPriority(89, 100);
 			}
 			tmpSnd = _enemy[_currEnemy].sound;
-			if(!smlayer_isSoundRunning(tmpSnd)) {
+			if (!smlayer_isSoundRunning(tmpSnd)) {
 				smlayer_startSfx(tmpSnd);
 				smlayer_soundSetPriority(tmpSnd, 100);
 			}
@@ -889,13 +900,13 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 		case 4:
 		case 5:
 			postCase3(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
-			if(!smlayer_isSoundRunning(88))
+			if (!smlayer_isSoundRunning(88))
 				smlayer_startSfx(88);
 			smlayer_soundSetPan(88, ((_actor[0].x+160)>>2)+64);
 			break;
 		case 6:
 			postCase5(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
-			if(!smlayer_isSoundRunning(88))
+			if (!smlayer_isSoundRunning(88))
 				smlayer_startSfx(88);
 			smlayer_soundSetPan(88, ((_actor[0].x+160)>>2)+64);
 			break;
@@ -921,12 +932,12 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 		case 13:
 			postCase12(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
 			needMore = true;
-			if(!smlayer_isSoundRunning(89)) {
+			if (!smlayer_isSoundRunning(89)) {
 				smlayer_startSfx(89);
 				smlayer_soundSetPriority(89, 100);
 			}
 			tmpSnd = _enemy[_currEnemy].sound;
-			if(!smlayer_isSoundRunning(tmpSnd)) {
+			if (!smlayer_isSoundRunning(tmpSnd)) {
 				smlayer_startSfx(tmpSnd);
 				smlayer_soundSetPriority(tmpSnd, 100);
 			}
@@ -934,7 +945,7 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 			smlayer_soundSetPan(tmpSnd, ((_actor[1].x+160)>>2)+64);
 			break;
 		case 24:
-			if(!smlayer_isSoundRunning(90)) {
+			if (!smlayer_isSoundRunning(90)) {
 				smlayer_startSfx(90);
 				smlayer_soundSetPriority(90, 100);
 			}
@@ -951,7 +962,7 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 
 		if (_currScenePropIdx)
 			postCaseAll(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
-	
+
 		_actor[0].frame++;
 		_actor[0].act[3].frame++;
 		_actor[0].act[2].frame++;
@@ -963,21 +974,21 @@ void Insane::procPostRendering(byte *renderBitmap, int32 codecparam, int32 setup
 		_actor[1].act[1].frame++;
 		_actor[1].act[0].frame++;
 	}
-	
+
 	if (!_val115_) {
 		smlayer_overrideDrawActorAt(&renderBitmap[0], renderBitmap[2], renderBitmap[3]);
 		_isBenCut = 0;
 	}
-	
+
 	if (_isBenCut)
 		smlayer_drawSomething(renderBitmap, codecparam, 89, 56, 1, _smush_bencutNut, 0, 0, 0);
-	
+
 	if (!_keyboardDisable)
 		_vm->processActors();
-	
+
 	if (needMore)
 		postCaseMore(renderBitmap, codecparam, setupsan12, setupsan13, curFrame, maxFrame);
-	
+
 	_lastKey = 0;
 	_tiresRustle = false;
 }
@@ -988,7 +999,7 @@ void Insane::postCase11(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 		if (_firstBattle) {
 			smush_setToFinish();
 		} else {
-			queueSceneSwitch(1, _smush_minedrivFlu, "minedriv.san", 64, 0, 
+			queueSceneSwitch(1, _smush_minedrivFlu, "minedriv.san", 64, 0,
 							 _continueFrame, 1300);
 		}
 	}
@@ -999,13 +1010,13 @@ void Insane::postCase11(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 void Insane::postCase0(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 					   int32 setupsan13, int32 curFrame, int32 maxFrame) {
 	turnBen(true);
-	
+
 	if (!curFrame || curFrame == 420)
 		smlayer_setFluPalette(_smush_roadrashRip, 0);
-	
+
 	if (curFrame >= maxFrame)
 		smush_rewindCurrentSan(1088, -1, -1);
-	
+
 	_roadBumps = false;
 	_roadBranch = false;
 	_roadStop = false;
@@ -1018,7 +1029,7 @@ void Insane::postCase17(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 						int32 setupsan13, int32 curFrame, int32 maxFrame) {
 	if (curFrame >= maxFrame && !_needSceneSwitch) {
 		if (_currSceneId == 18) {
-			queueSceneSwitch(17, _smush_minedrivFlu, "minedriv.san", 64, 0, 
+			queueSceneSwitch(17, _smush_minedrivFlu, "minedriv.san", 64, 0,
 							 _continueFrame1, 1300);
 			writeArray(9, 1);
 		} else {
@@ -1035,31 +1046,31 @@ void Insane::postCase16(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 						int32 setupsan13, int32 curFrame, int32 maxFrame) {
 	char buf[12];
 	int32 tmp;
-	
+
 	turnBen(true);
 	sprintf(buf, "^f01%02o", curFrame & 0x3f);
 	smlayer_showStatusMsg(-1, renderBitmap, codecparam, 180, 168, 1, 2, 0, "%s", buf);
 	tmp = 400-curFrame;
-	
+
 	if (tmp < 0)
 		tmp += 1300;
-	
+
 	sprintf(buf, "^f01%04d", tmp);
 	smlayer_showStatusMsg(-1, renderBitmap, codecparam, 202, 168, 1, 2, 0, "%s", buf);
-	
+
 	sprintf(buf, "^f01%02o", curFrame & 0xff);
 	smlayer_showStatusMsg(-1, renderBitmap, codecparam, 240, 168, 1, 2, 0, "%s", buf);
 	smlayer_showStatusMsg(-1, renderBitmap, codecparam, 170, 43, 1, 2, 0, "%s", buf);
 
 	smlayer_drawSomething(renderBitmap, codecparam, 0, 0, 1, _smush_bensgoggNut, 0, 0, 0);
-	
+
 	if (!_objectDetected)
-		smlayer_drawSomething(renderBitmap, codecparam, 24, 170, 1, 
+		smlayer_drawSomething(renderBitmap, codecparam, 24, 170, 1,
 							  _smush_iconsNut, 23, 0, 0);
-	
+
 	if (!curFrame)
 		smlayer_setFluPalette(_smush_goglpaltRip, 0);
-	
+
 	if (curFrame >= maxFrame) {
 		smush_rewindCurrentSan(1088, -1, -1);
 		smlayer_setFluPalette(_smush_goglpaltRip, 0);
@@ -1081,7 +1092,7 @@ void Insane::postCase1(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 
 	if ((curFrame >= maxFrame) && !_needSceneSwitch) {
 		flu = &_fluConf[14 + _iactSceneId2];
-		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0, 
+		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0,
 						 flu->startFrame, flu->numFrames);
 	}
 	_roadBranch = false;
@@ -1092,7 +1103,7 @@ void Insane::postCase2(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 					   int32 setupsan13, int32 curFrame, int32 maxFrame) {
 	turnBen(_battleScene != 0);
 	turnEnemy(true);
-	
+
 	if (!curFrame)
 		smlayer_setFluPalette(_smush_roadrashRip, 0);
 
@@ -1109,10 +1120,10 @@ void Insane::postCase20(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 						int32 setupsan13, int32 curFrame, int32 maxFrame) {
 	turnBen(true);
 	turnEnemy(true);
-	
+
 	if (curFrame >= maxFrame)
 		smush_rewindCurrentSan(1088, -1, -1);
-	
+
 	_roadBumps = false;
 	_roadBranch = false;
 	_roadStop = false;
@@ -1121,8 +1132,11 @@ void Insane::postCase20(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 
 void Insane::postCase3(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 					   int32 setupsan13, int32 curFrame, int32 maxFrame) {
-	turnBen(true);
-	
+	if ((_vm->_features & GF_DEMO) && (_vm->_platform == Common::kPlatformPC))
+		turnBen(false);
+	else
+		turnBen(true);
+
 	if (_actor[0].x >= 158 && _actor[0].x <= 168) {
 		if (!smlayer_isSoundRunning(86))
 			smlayer_startSfx(86);
@@ -1189,7 +1203,7 @@ void Insane::postCase5(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 			}
 		}
 	}
-	
+
 	_carIsBroken = false;
 	_roadStop = false;
 	_roadBranch = false;
@@ -1206,7 +1220,7 @@ void Insane::postCase6(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 		else
 			flu = &_fluConf[0 + _iactSceneId2];
 
-		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0, 
+		queueSceneSwitch(flu->sceneId, *flu->fluPtr, flu->filenamePtr, 64, 0,
 						 flu->startFrame, flu->numFrames);
 	}
 	_roadBranch = false;
@@ -1230,7 +1244,7 @@ void Insane::postCase8(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 			}
 		}
  	}
-	
+
 	_roadBranch = false;
 	_roadStop = false;
 }
@@ -1291,7 +1305,7 @@ void Insane::postCase10(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 			}
 		}
  	}
-	
+
 	_roadBranch = false;
 	_roadStop = false;
 }
@@ -1481,3 +1495,4 @@ void Insane::postCaseMore(byte *renderBitmap, int32 codecparam, int32 setupsan12
 }
 
 }
+
